@@ -139,6 +139,18 @@ export USE_FFMPEG=1
 
 # The following symbols do not exist on the minimal macOS version (10.7), so they are disabled
 # here. This allows compilation also with newer macOS SDKs.
+
+# Added symbols in macOS 10.15 / iOS 13 / tvOS 13
+export ac_cv_func_aligned_alloc=no
+export ac_cv_func_timespec_get=no
+
+# Added symbol in macOS 10.14 / iOS 12 / tvOS 9
+export ac_cv_func_thread_get_register_pointer_values=no
+
+# Added symbols in macOS 10.13 / iOS 11 / tvOS 11
+export ac_cv_func_utimensat=no
+export ac_cv_func_futimens=no
+
 # Added symbols in 10.13
 export ac_cv_func_open_wmemstream=no
 export ac_cv_func_fmemopen=no
@@ -155,6 +167,7 @@ export ac_cv_func_dirname_r=no
 export ac_cv_func_getentropy=no
 export ac_cv_func_mkostemp=no
 export ac_cv_func_mkostemps=no
+export ac_cv_func_timingsafe_bcmp=no
 
 # Added symbols between 10.7 and 10.11
 export ac_cv_func_ffsll=no
