@@ -2,7 +2,7 @@
  * attachment.c: Input reading an attachment.
  *****************************************************************************
  * Copyright (C) 2009 Laurent Aimar
- * $Id: 1a320ff935073bcbed540daf19a9a6fe14d9348e $
+ * $Id: 9623f823693dca6d5ef894b5b1a0a6ec42173843 $
  *
  * Authors: Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
  *
@@ -54,9 +54,9 @@ vlc_module_end()
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static ssize_t Read(access_t *, uint8_t *, size_t);
-static int     Seek(access_t *, uint64_t);
-static int     Control(access_t *, int, va_list);
+static ssize_t Read(stream_t *, void *, size_t);
+static int     Seek(stream_t *, uint64_t);
+static int     Control(stream_t *, int, va_list);
 
 struct access_sys_t
 {
@@ -67,13 +67,13 @@ struct access_sys_t
 /* */
 static int Open(vlc_object_t *object)
 {
-    access_t     *access = (access_t *)object;
+    stream_t     *access = (stream_t *)object;
 
     input_thread_t *input = access->p_input;
     if (!input)
         return VLC_EGENERIC;
 
-    access_sys_t *sys = malloc(sizeof (*sys));
+    access_sys_t *sys = vlc_obj_malloc(object, sizeof (*sys));
     if (unlikely(sys == NULL))
         return VLC_ENOMEM;
 
@@ -84,14 +84,12 @@ static int Open(vlc_object_t *object)
     if (sys->attachment == NULL) {
         msg_Err(access, "Failed to find the attachment '%s'",
                 access->psz_location);
-        free(sys);
         return VLC_EGENERIC;
     }
 
     sys->offset = 0;
 
     /* */
-    access_InitFields(access);
     access->pf_read    = Read;
     access->pf_block   = NULL;
     access->pf_control = Control;
@@ -103,21 +101,19 @@ static int Open(vlc_object_t *object)
 /* */
 static void Close(vlc_object_t *object)
 {
-    access_t     *access = (access_t *)object;
+    stream_t     *access = (stream_t *)object;
     access_sys_t *sys = access->p_sys;
 
     vlc_input_attachment_Delete(sys->attachment);
-    free(sys);
 }
 
 /* */
-static ssize_t Read(access_t *access, uint8_t *buffer, size_t size)
+static ssize_t Read(stream_t *access, void *buffer, size_t size)
 {
     access_sys_t *sys = access->p_sys;
     input_attachment_t *a = sys->attachment;
 
-    access->info.b_eof = sys->offset >= (uint64_t)a->i_data;
-    if (access->info.b_eof)
+    if (sys->offset >= (uint64_t)a->i_data)
         return 0;
 
     const size_t copy = __MIN(size, a->i_data - sys->offset);
@@ -127,7 +123,7 @@ static ssize_t Read(access_t *access, uint8_t *buffer, size_t size)
 }
 
 /* */
-static int Seek(access_t *access, uint64_t position)
+static int Seek(stream_t *access, uint64_t position)
 {
     access_sys_t *sys = access->p_sys;
     input_attachment_t *a = sys->attachment;
@@ -136,30 +132,29 @@ static int Seek(access_t *access, uint64_t position)
         position = a->i_data;
 
     sys->offset = position;
-    access->info.b_eof = false;
     return VLC_SUCCESS;
 }
 
 /* */
-static int Control(access_t *access, int query, va_list args)
+static int Control(stream_t *access, int query, va_list args)
 {
     access_sys_t *sys = access->p_sys;
 
     switch (query)
     {
-    case ACCESS_CAN_SEEK:
-    case ACCESS_CAN_FASTSEEK:
-    case ACCESS_CAN_PAUSE:
-    case ACCESS_CAN_CONTROL_PACE:
+    case STREAM_CAN_SEEK:
+    case STREAM_CAN_FASTSEEK:
+    case STREAM_CAN_PAUSE:
+    case STREAM_CAN_CONTROL_PACE:
         *va_arg(args, bool *) = true;
         break;
-    case ACCESS_GET_SIZE:
+    case STREAM_GET_SIZE:
         *va_arg(args, uint64_t *) = sys->attachment->i_data;
         break;
-    case ACCESS_GET_PTS_DELAY:
+    case STREAM_GET_PTS_DELAY:
         *va_arg(args, int64_t *) = DEFAULT_PTS_DELAY;
         break;
-    case ACCESS_SET_PAUSE_STATE:
+    case STREAM_SET_PAUSE_STATE:
         return VLC_SUCCESS;
 
     default:
