@@ -63,18 +63,17 @@ struct frontend_t
 #define FRONTEND_LOCK_TIMEOUT 10000000 /* 10 s */
 
 /* Local prototypes */
-static int FrontendInfo( access_t * );
-static int FrontendSetQPSK( access_t * );
-static int FrontendSetQAM( access_t * );
-static int FrontendSetOFDM( access_t * );
-static int FrontendSetATSC( access_t * );
+static int FrontendInfo( vlc_object_t *, dvb_sys_t * );
+static int FrontendSetQPSK( vlc_object_t *, dvb_sys_t * );
+static int FrontendSetQAM( vlc_object_t *, dvb_sys_t * );
+static int FrontendSetOFDM( vlc_object_t *, dvb_sys_t * );
+static int FrontendSetATSC( vlc_object_t *, dvb_sys_t * );
 
 /*****************************************************************************
  * FrontendOpen : Determine frontend device information and capabilities
  *****************************************************************************/
-int FrontendOpen( access_t *p_access )
+int FrontendOpen( vlc_object_t *p_access, dvb_sys_t *p_sys, const char *psz_access )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     frontend_t * p_frontend;
     unsigned int i_adapter, i_device;
     bool b_probe;
@@ -108,9 +107,9 @@ int FrontendOpen( access_t *p_access )
         const char * psz_expected = NULL;
         const char * psz_real;
 
-        if( FrontendInfo( p_access ) < 0 )
+        if( FrontendInfo( p_access, p_sys ) < 0 )
         {
-            close( p_sys->i_frontend_handle );
+            vlc_close( p_sys->i_frontend_handle );
             free( p_frontend );
             return VLC_EGENERIC;
         }
@@ -134,28 +133,28 @@ int FrontendOpen( access_t *p_access )
         }
 
         /* Sanity checks */
-        if( (!strncmp( p_access->psz_access, "qpsk", 4 ) ||
-             !strncmp( p_access->psz_access, "dvb-s", 5 ) ||
-             !strncmp( p_access->psz_access, "satellite", 9 ) ) &&
+        if( (!strncmp( psz_access, "qpsk", 4 ) ||
+             !strncmp( psz_access, "dvb-s", 5 ) ||
+             !strncmp( psz_access, "satellite", 9 ) ) &&
              (p_frontend->info.type != FE_QPSK) )
         {
             psz_expected = "DVB-S";
         }
-        if( (!strncmp( p_access->psz_access, "cable", 5 ) ||
-             !strncmp( p_access->psz_access, "dvb-c", 5 ) ) &&
+        if( (!strncmp( psz_access, "cable", 5 ) ||
+             !strncmp( psz_access, "dvb-c", 5 ) ) &&
              (p_frontend->info.type != FE_QAM) )
         {
             psz_expected = "DVB-C";
         }
-        if( (!strncmp( p_access->psz_access, "terrestrial", 11 ) ||
-             !strncmp( p_access->psz_access, "dvb-t", 5 ) ) &&
+        if( (!strncmp( psz_access, "terrestrial", 11 ) ||
+             !strncmp( psz_access, "dvb-t", 5 ) ) &&
              (p_frontend->info.type != FE_OFDM) )
         {
             psz_expected = "DVB-T";
         }
 
-        if( (!strncmp( p_access->psz_access, "usdigital", 9 ) ||
-             !strncmp( p_access->psz_access, "atsc", 4 ) ) &&
+        if( (!strncmp( psz_access, "usdigital", 9 ) ||
+             !strncmp( psz_access, "atsc", 4 ) ) &&
              (p_frontend->info.type != FE_ATSC) )
         {
             psz_expected = "ATSC";
@@ -165,7 +164,7 @@ int FrontendOpen( access_t *p_access )
         {
             msg_Err( p_access, "requested type %s not supported by %s tuner",
                      psz_expected, psz_real );
-            close( p_sys->i_frontend_handle );
+            vlc_close( p_sys->i_frontend_handle );
             free( p_frontend );
             return VLC_EGENERIC;
         }
@@ -174,19 +173,19 @@ int FrontendOpen( access_t *p_access )
     {
         msg_Dbg( p_access, "using default values for frontend info" );
 
-        msg_Dbg( p_access, "method of access is %s", p_access->psz_access );
+        msg_Dbg( p_access, "method of access is %s", psz_access );
         p_frontend->info.type = FE_QPSK;
-        if( !strncmp( p_access->psz_access, "qpsk", 4 ) ||
-            !strncmp( p_access->psz_access, "dvb-s", 5 ) )
+        if( !strncmp( psz_access, "qpsk", 4 ) ||
+            !strncmp( psz_access, "dvb-s", 5 ) )
             p_frontend->info.type = FE_QPSK;
-        else if( !strncmp( p_access->psz_access, "cable", 5 ) ||
-                 !strncmp( p_access->psz_access, "dvb-c", 5 ) )
+        else if( !strncmp( psz_access, "cable", 5 ) ||
+                 !strncmp( psz_access, "dvb-c", 5 ) )
             p_frontend->info.type = FE_QAM;
-        else if( !strncmp( p_access->psz_access, "terrestrial", 11 ) ||
-                 !strncmp( p_access->psz_access, "dvb-t", 5 ) )
+        else if( !strncmp( psz_access, "terrestrial", 11 ) ||
+                 !strncmp( psz_access, "dvb-t", 5 ) )
             p_frontend->info.type = FE_OFDM;
-        else if( !strncmp( p_access->psz_access, "usdigital", 9 ) ||
-                 !strncmp( p_access->psz_access, "atsc", 4 ) )
+        else if( !strncmp( psz_access, "usdigital", 9 ) ||
+                 !strncmp( psz_access, "atsc", 4 ) )
             p_frontend->info.type = FE_ATSC;
     }
 
@@ -196,13 +195,12 @@ int FrontendOpen( access_t *p_access )
 /*****************************************************************************
  * FrontendClose : Close the frontend
  *****************************************************************************/
-void FrontendClose( access_t *p_access )
+void FrontendClose( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
-
+    VLC_UNUSED(p_access);
     if( p_sys->p_frontend )
     {
-        close( p_sys->i_frontend_handle );
+        vlc_close( p_sys->i_frontend_handle );
         free( p_sys->p_frontend );
 
         p_sys->p_frontend = NULL;
@@ -212,15 +210,13 @@ void FrontendClose( access_t *p_access )
 /*****************************************************************************
  * FrontendSet : Tune !
  *****************************************************************************/
-int FrontendSet( access_t *p_access )
+int FrontendSet( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
-
     switch( p_sys->p_frontend->info.type )
     {
     /* DVB-S */
     case FE_QPSK:
-        if( FrontendSetQPSK( p_access ) )
+        if( FrontendSetQPSK( p_access, p_sys ) )
         {
             msg_Err( p_access, "DVB-S tuning error" );
             return VLC_EGENERIC;
@@ -229,7 +225,7 @@ int FrontendSet( access_t *p_access )
 
     /* DVB-C */
     case FE_QAM:
-        if( FrontendSetQAM( p_access ) )
+        if( FrontendSetQAM( p_access, p_sys ) )
         {
             msg_Err( p_access, "DVB-C tuning error" );
             return VLC_EGENERIC;
@@ -238,7 +234,7 @@ int FrontendSet( access_t *p_access )
 
     /* DVB-T */
     case FE_OFDM:
-        if( FrontendSetOFDM( p_access ) )
+        if( FrontendSetOFDM( p_access, p_sys ) )
         {
             msg_Err( p_access, "DVB-T tuning error" );
             return VLC_EGENERIC;
@@ -247,7 +243,7 @@ int FrontendSet( access_t *p_access )
 
     /* ATSC */
     case FE_ATSC:
-        if( FrontendSetATSC( p_access ) )
+        if( FrontendSetATSC( p_access, p_sys ) )
         {
             msg_Err( p_access, "ATSC tuning error" );
             return VLC_EGENERIC;
@@ -260,16 +256,14 @@ int FrontendSet( access_t *p_access )
         return VLC_EGENERIC;
     }
     p_sys->p_frontend->i_last_status = 0;
-    p_sys->i_frontend_timeout = mdate() + FRONTEND_LOCK_TIMEOUT;
     return VLC_SUCCESS;
 }
 
 /*****************************************************************************
  * FrontendPoll : Poll for frontend events
  *****************************************************************************/
-void FrontendPoll( access_t *p_access )
+void FrontendPoll( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     frontend_t * p_frontend = p_sys->p_frontend;
     struct dvb_frontend_event event;
     fe_status_t i_status, i_diff;
@@ -320,10 +314,9 @@ void FrontendPoll( access_t *p_access )
                 frontend_statistic_t stat;
 
                 msg_Dbg( p_access, "frontend has acquired lock" );
-                p_sys->i_frontend_timeout = 0;
 
                 /* Read some statistics */
-                if( !FrontendGetStatistic( p_access, &stat ) )
+                if( !FrontendGetStatistic( p_sys, &stat ) )
                 {
                     if( stat.i_ber >= 0 )
                         msg_Dbg( p_access, "- Bit error rate: %d", stat.i_ber );
@@ -336,23 +329,21 @@ void FrontendPoll( access_t *p_access )
             else
             {
                 msg_Dbg( p_access, "frontend has lost lock" );
-                p_sys->i_frontend_timeout = mdate() + FRONTEND_LOCK_TIMEOUT;
             }
 
             IF_UP( FE_REINIT )
             {
                 /* The frontend was reinited. */
                 msg_Warn( p_access, "reiniting frontend");
-                FrontendSet( p_access );
+                FrontendSet( p_access, p_sys );
             }
         }
 #undef IF_UP
     }
 }
 
-int FrontendGetStatistic( access_t *p_access, frontend_statistic_t *p_stat )
+int FrontendGetStatistic( dvb_sys_t *p_sys, frontend_statistic_t *p_stat )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     frontend_t * p_frontend = p_sys->p_frontend;
 
     if( (p_frontend->i_last_status & FE_HAS_LOCK) == 0 )
@@ -369,9 +360,8 @@ int FrontendGetStatistic( access_t *p_access, frontend_statistic_t *p_stat )
     return VLC_SUCCESS;
 }
 
-void FrontendGetStatus( access_t *p_access, frontend_status_t *p_status )
+void FrontendGetStatus( dvb_sys_t *p_sys, frontend_status_t *p_status )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     frontend_t * p_frontend = p_sys->p_frontend;
 
     p_status->b_has_signal = (p_frontend->i_last_status & FE_HAS_SIGNAL) != 0;
@@ -379,94 +369,82 @@ void FrontendGetStatus( access_t *p_access, frontend_status_t *p_status )
     p_status->b_has_lock = (p_frontend->i_last_status & FE_HAS_LOCK) != 0;
 }
 
-static int ScanParametersDvbS( access_t *p_access, scan_parameter_t *p_scan )
+static int ScanParametersDvbS( vlc_object_t *p_access, dvb_sys_t *p_sys, scan_parameter_t *p_scan )
 {
-    const frontend_t *p_frontend = p_access->p_sys->p_frontend;
+    const frontend_t *p_frontend = p_sys->p_frontend;
 
-    memset( p_scan, 0, sizeof(*p_scan) );
     p_scan->type = SCAN_DVB_S;
 
     p_scan->frequency.i_min = p_frontend->info.frequency_min;
     p_scan->frequency.i_max = p_frontend->info.frequency_max;
     /* set satellite config file path */
-    p_scan->sat_info.psz_name = var_InheritString( p_access, "dvb-satellite" );
+    char *psz_name = var_InheritString( p_access, "dvb-satellite" );
+    if( psz_name )
+    {
+        char *data_dir = config_GetDataDir();
+        if( !data_dir || -1 ==  asprintf( &p_scan->psz_scanlist_file,
+            "%s" DIR_SEP "dvb" DIR_SEP "dvb-s" DIR_SEP "%s", data_dir, psz_name ) )
+        {
+            p_scan->psz_scanlist_file = NULL;
+        }
+        p_scan->scanlist_format = FORMAT_DVBv3;
+        free( data_dir );
+        free( psz_name );
+    }
 
     return VLC_SUCCESS;
 }
 
-static int ScanParametersDvbC( access_t *p_access, scan_parameter_t *p_scan )
+static int ScanParametersDvbC( vlc_object_t *p_access, dvb_sys_t *p_sys, scan_parameter_t *p_scan )
 {
-    const frontend_t *p_frontend = p_access->p_sys->p_frontend;
+    const frontend_t *p_frontend = p_sys->p_frontend;
 
-    memset( p_scan, 0, sizeof(*p_scan) );
     p_scan->type = SCAN_DVB_C;
     p_scan->b_exhaustive = false;
 
     /* */
     p_scan->frequency.i_min = p_frontend->info.frequency_min;
     p_scan->frequency.i_max = p_frontend->info.frequency_max;
-    p_scan->frequency.i_step = p_frontend->info.frequency_stepsize
-        ? p_frontend->info.frequency_stepsize : 166667;
-    p_scan->frequency.i_count = (p_scan->frequency.i_max-p_scan->frequency.i_min)/p_scan->frequency.i_step;
+    p_scan->frequency.i_step = p_frontend->info.frequency_stepsize;
 
     /* if frontend can do auto, don't scan them */
-    if( p_frontend->info.caps & FE_CAN_QAM_AUTO )
-    {
-        p_scan->b_modulation_set = true;
-    } else {
-        p_scan->b_modulation_set = false;
-        /* our scanning code flips modulation from 16..256 automaticly*/
-        p_scan->i_modulation = 0;
-    }
+    p_scan->b_modulation_set = ( p_frontend->info.caps & FE_CAN_QAM_AUTO );
+    /* our scanning code flips modulation from 16..256 automaticly*/
 
     /* if user supplies symbolrate, don't scan those */
-    if( var_GetInteger( p_access, "dvb-srate" ) )
-        p_scan->b_symbolrate_set = true;
-    else
-        p_scan->b_symbolrate_set = false;
+    p_scan->i_symbolrate = var_GetInteger( p_access, "dvb-srate" );
 
-    /* */
-    p_scan->bandwidth.i_min  = 6;
-    p_scan->bandwidth.i_max  = 8;
-    p_scan->bandwidth.i_step = 1;
-    p_scan->bandwidth.i_count = 3;
     return VLC_SUCCESS;
 }
 
-static int ScanParametersDvbT( access_t *p_access, scan_parameter_t *p_scan )
+static int ScanParametersDvbT( vlc_object_t *p_access, dvb_sys_t *p_sys, scan_parameter_t *p_scan )
 {
-    const frontend_t *p_frontend = p_access->p_sys->p_frontend;
+    const frontend_t *p_frontend = p_sys->p_frontend;
 
-    memset( p_scan, 0, sizeof(*p_scan) );
     p_scan->type = SCAN_DVB_T;
     p_scan->b_exhaustive = false;
 
+    p_scan->psz_scanlist_file = var_InheritString( p_access, "dvb-scanlist" );
+    p_scan->scanlist_format = FORMAT_DVBv5;
+
     /* */
     p_scan->frequency.i_min = p_frontend->info.frequency_min;
     p_scan->frequency.i_max = p_frontend->info.frequency_max;
-    p_scan->frequency.i_step = p_frontend->info.frequency_stepsize
-        ? p_frontend->info.frequency_stepsize : 166667;
-    p_scan->frequency.i_count = (p_scan->frequency.i_max-p_scan->frequency.i_min)/p_scan->frequency.i_step;
+    p_scan->frequency.i_step = p_frontend->info.frequency_stepsize;
 
-    /* */
-    p_scan->bandwidth.i_min  = 6;
-    p_scan->bandwidth.i_max  = 8;
-    p_scan->bandwidth.i_step = 1;
-    p_scan->bandwidth.i_count = 3;
     return VLC_SUCCESS;
 }
 
-int  FrontendGetScanParameter( access_t *p_access, scan_parameter_t *p_scan )
+int FrontendFillScanParameter( vlc_object_t *p_access, dvb_sys_t *p_sys, scan_parameter_t *p_scan )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     const frontend_t *p_frontend = p_sys->p_frontend;
 
     if( p_frontend->info.type == FE_OFDM )              /* DVB-T */
-        return ScanParametersDvbT( p_access, p_scan );
+        return ScanParametersDvbT( p_access, p_sys, p_scan );
     else if( p_frontend->info.type == FE_QAM )          /* DVB-C */
-        return ScanParametersDvbC( p_access, p_scan );
+        return ScanParametersDvbC( p_access, p_sys, p_scan );
     else if( p_frontend->info.type == FE_QPSK )
-        return ScanParametersDvbS( p_access, p_scan );  /* DVB-S */
+        return ScanParametersDvbS( p_access, p_sys, p_scan );  /* DVB-S */
 
     msg_Err( p_access, "frontend scanning not supported" );
     return VLC_EGENERIC;
@@ -475,9 +453,8 @@ int  FrontendGetScanParameter( access_t *p_access, scan_parameter_t *p_scan )
 /*****************************************************************************
  * FrontendInfo : Return information about given frontend
  *****************************************************************************/
-static int FrontendInfo( access_t *p_access )
+static int FrontendInfo( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     frontend_t *p_frontend = p_sys->p_frontend;
 
     /* Determine type of frontend */
@@ -507,10 +484,10 @@ static int FrontendInfo( access_t *p_access )
             break;
 #if 0 /* DVB_API_VERSION == 3 */
         case FE_MEMORY:
-            msg_Dbg(p_access, "  type = MEMORY" );
+            msg_Dbg( p_access, "  type = MEMORY" );
             break;
         case FE_NET:
-            msg_Dbg(p_access, "  type = NETWORK" );
+            msg_Dbg( p_access, "  type = NETWORK" );
             break;
 #endif
         default:
@@ -609,7 +586,7 @@ static int FrontendInfo( access_t *p_access )
 /*****************************************************************************
  * Decoding the DVB parameters (common)
  *****************************************************************************/
-static fe_spectral_inversion_t DecodeInversion( access_t *p_access )
+static fe_spectral_inversion_t DecodeInversion( vlc_object_t *p_access )
 {
     int i_val;
     fe_spectral_inversion_t fe_inversion = 0;
@@ -633,7 +610,7 @@ static fe_spectral_inversion_t DecodeInversion( access_t *p_access )
 /*****************************************************************************
  * FrontendSetQPSK : controls the FE device
  *****************************************************************************/
-static fe_sec_voltage_t DecodeVoltage( access_t *p_access )
+static fe_sec_voltage_t DecodeVoltage( vlc_object_t *p_access )
 {
     switch( var_GetInteger( p_access, "dvb-voltage" ) )
     {
@@ -644,7 +621,7 @@ static fe_sec_voltage_t DecodeVoltage( access_t *p_access )
     }
 }
 
-static fe_sec_tone_mode_t DecodeTone( access_t *p_access )
+static fe_sec_tone_mode_t DecodeTone( vlc_object_t *p_access )
 {
     switch( var_GetInteger( p_access, "dvb-tone" ) )
     {
@@ -660,9 +637,8 @@ struct diseqc_cmd_t
     uint32_t wait;
 };
 
-static int DoDiseqc( access_t *p_access )
+static int DoDiseqc( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     int i_val;
     bool b_val;
     int i_frequency, i_lnb_slof;
@@ -763,9 +739,8 @@ static int DoDiseqc( access_t *p_access )
     return 0;
 }
 
-static int FrontendSetQPSK( access_t *p_access )
+static int FrontendSetQPSK( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     struct dvb_frontend_parameters fep;
     int i_val;
     int i_frequency, i_lnb_slof = 0, i_lnb_lof1, i_lnb_lof2 = 0;
@@ -842,7 +817,7 @@ static int FrontendSetQPSK( access_t *p_access )
 
     fep.u.qpsk.fec_inner = FEC_NONE;
 
-    if( DoDiseqc( p_access ) < 0 )
+    if( DoDiseqc( p_access, p_sys ) < 0 )
     {
         return VLC_EGENERIC;
     }
@@ -869,9 +844,8 @@ static int FrontendSetQPSK( access_t *p_access )
 /*****************************************************************************
  * FrontendSetQAM : controls the FE device
  *****************************************************************************/
-static int FrontendSetQAM( access_t *p_access )
+static int FrontendSetQAM( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     frontend_t *p_frontend = p_sys->p_frontend;
     struct dvb_frontend_parameters fep;
     unsigned int i_val;
@@ -917,7 +891,7 @@ static int FrontendSetQAM( access_t *p_access )
 /*****************************************************************************
  * FrontendSetOFDM : controls the FE device
  *****************************************************************************/
-static fe_bandwidth_t DecodeBandwidth( access_t *p_access )
+static fe_bandwidth_t DecodeBandwidth( vlc_object_t *p_access )
 {
     fe_bandwidth_t      fe_bandwidth = 0;
     int i_bandwidth = var_GetInteger( p_access, "dvb-bandwidth" );
@@ -938,7 +912,7 @@ static fe_bandwidth_t DecodeBandwidth( access_t *p_access )
     return fe_bandwidth;
 }
 
-static fe_transmit_mode_t DecodeTransmission( access_t *p_access )
+static fe_transmit_mode_t DecodeTransmission( vlc_object_t *p_access )
 {
     fe_transmit_mode_t  fe_transmission = 0;
     int i_transmission = var_GetInteger( p_access, "dvb-transmission" );
@@ -958,7 +932,7 @@ static fe_transmit_mode_t DecodeTransmission( access_t *p_access )
     return fe_transmission;
 }
 
-static fe_hierarchy_t DecodeHierarchy( access_t *p_access )
+static fe_hierarchy_t DecodeHierarchy( vlc_object_t *p_access )
 {
     fe_hierarchy_t      fe_hierarchy = 0;
     int i_hierarchy = var_GetInteger( p_access, "dvb-hierarchy" );
@@ -980,9 +954,8 @@ static fe_hierarchy_t DecodeHierarchy( access_t *p_access )
     return fe_hierarchy;
 }
 
-static int FrontendSetOFDM( access_t * p_access )
+static int FrontendSetOFDM( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     struct dvb_frontend_parameters fep;
 
     /* Prepare the fep structure */
@@ -1021,9 +994,8 @@ static int FrontendSetOFDM( access_t * p_access )
 /*****************************************************************************
  * FrontendSetATSC : controls the FE device
  *****************************************************************************/
-static int FrontendSetATSC( access_t *p_access )
+static int FrontendSetATSC( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     struct dvb_frontend_parameters fep;
 
     /* Prepare the fep structure */
@@ -1058,7 +1030,7 @@ static int FrontendSetATSC( access_t *p_access )
 /*****************************************************************************
  * DMXSetFilter : controls the demux to add a filter
  *****************************************************************************/
-int DMXSetFilter( access_t * p_access, int i_pid, int * pi_fd, int i_type )
+int DMXSetFilter( vlc_object_t *p_access, int i_pid, int * pi_fd, int i_type )
 {
     struct dmx_pes_filter_params s_filter_params;
     unsigned int i_adapter, i_device;
@@ -1194,7 +1166,7 @@ int DMXSetFilter( access_t * p_access, int i_pid, int * pi_fd, int i_type )
 /*****************************************************************************
  * DMXUnsetFilter : removes a filter
  *****************************************************************************/
-int DMXUnsetFilter( access_t * p_access, int i_fd )
+int DMXUnsetFilter( vlc_object_t *p_access, int i_fd )
 {
     if( ioctl( i_fd, DMX_STOP ) < 0 )
     {
@@ -1204,7 +1176,7 @@ int DMXUnsetFilter( access_t * p_access, int i_fd )
     }
 
     msg_Dbg( p_access, "DMXUnsetFilter: closing demux %d", i_fd );
-    close( i_fd );
+    vlc_close( i_fd );
     return VLC_SUCCESS;
 }
 
@@ -1216,9 +1188,8 @@ int DMXUnsetFilter( access_t * p_access, int i_fd )
 /*****************************************************************************
  * DVROpen :
  *****************************************************************************/
-int DVROpen( access_t * p_access )
+int DVROpen( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
     unsigned int i_adapter, i_device;
     char dvr[128];
 
@@ -1252,9 +1223,8 @@ int DVROpen( access_t * p_access )
 /*****************************************************************************
  * DVRClose :
  *****************************************************************************/
-void DVRClose( access_t * p_access )
+void DVRClose( vlc_object_t *p_access, dvb_sys_t *p_sys )
 {
-    access_sys_t *p_sys = p_access->p_sys;
-
-    close( p_sys->i_handle );
+    VLC_UNUSED(p_access);
+    vlc_close( p_sys->i_handle );
 }
