@@ -2,7 +2,7 @@
  * dvdread.c : DvdRead input module for vlc
  *****************************************************************************
  * Copyright (C) 2001-2006 VLC authors and VideoLAN
- * $Id: daaa1efb630abaec7adcf048bd186cdae4051bc6 $
+ * $Id: 442b3a0e4902ee25318564204b4ea455549513bc $
  *
  * Authors: Stéphane Borel <stef@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -61,6 +61,7 @@
 #include <dvdread/nav_print.h>
 
 #include <assert.h>
+#include <limits.h>
 
 /*****************************************************************************
  * Module descriptor
@@ -152,10 +153,17 @@ static void DemuxTitles( demux_t *, int * );
 static void ESNew( demux_t *, int, int );
 
 static int  DvdReadSetArea  ( demux_t *, int, int, int );
-static void DvdReadSeek     ( demux_t *, int );
+static int  DvdReadSeek     ( demux_t *, int );
 static void DvdReadHandleDSI( demux_t *, uint8_t * );
 static void DvdReadFindCell ( demux_t * );
 
+#if DVDREAD_VERSION >= DVDREAD_VERSION_CODE(6, 1, 0)
+static void DvdReadLog( void *foo, dvd_logger_level_t i, const char *p, va_list z )
+{
+    demux_t *p_demux = (demux_t*)foo;
+    msg_GenericVa( p_demux, i, p, z );
+}
+#endif
 /*****************************************************************************
  * Open:
  *****************************************************************************/
@@ -192,14 +200,27 @@ static int Open( vlc_object_t *p_this )
 
     /* Open dvdread */
     const char *psz_path = ToLocale( psz_file );
+#if DVDREAD_VERSION >= DVDREAD_VERSION_CODE(6, 1, 0)
+    dvd_logger_cb cbs;
+    cbs.pf_log = DvdReadLog;
+    dvd_reader_t *p_dvdread = DVDOpen2( p_demux, &cbs, psz_path );
+#else
     dvd_reader_t *p_dvdread = DVDOpen( psz_path );
-
+#endif
     LocaleFree( psz_path );
     if( p_dvdread == NULL )
     {
         msg_Err( p_demux, "DVDRead cannot open source: %s", psz_file );
+#ifdef __APPLE__
+        vlc_dialog_display_error( p_demux, _("Problem accessing a system resource"),
+            _("Potentially, macOS blocks access to your disc. "
+              "Please open \"System Preferences\" -> \"Security & Privacy\" "
+              "and allow VLC to access your external media in \"Files and Folders\" section."));
+#else
         vlc_dialog_display_error( p_demux, _("Playback failure"),
                       _("DVDRead could not open the disc \"%s\"."), psz_file );
+#endif
+
         free( psz_file );
         return VLC_EGENERIC;
     }
@@ -359,9 +380,7 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
         {
             f = va_arg( args, double );
 
-            DvdReadSeek( p_demux, f * p_sys->i_title_blocks );
-
-            return VLC_SUCCESS;
+            return DvdReadSeek( p_demux, f * p_sys->i_title_blocks );
         }
         case DEMUX_GET_TIME:
             pi64 = va_arg( args, int64_t * );
@@ -483,6 +502,7 @@ static int Demux( demux_t *p_demux )
         /* Basic check to be sure we don't have a empty title
          * go to next title if so */
         //assert( p_buffer[41] == 0xbf && p_buffer[1027] == 0xbf );
+        DemuxBlock( p_demux, p_buffer, DVD_VIDEO_LB_LEN );
 
         /* Parse the contained dsi packet */
         DvdReadHandleDSI( p_demux, p_buffer );
@@ -508,12 +528,8 @@ static int Demux( demux_t *p_demux )
             return 0; /* EOF */
         }
 
-        /* FIXME: Ugly kludge: we send the pack block to the input for it
-         * sometimes has a zero scr and restart the sync */
         p_sys->i_cur_block++;
         p_sys->i_title_offset++;
-
-        DemuxBlock( p_demux, p_buffer, DVD_VIDEO_LB_LEN );
     }
 
     if( p_sys->i_cur_cell >= p_sys->p_cur_pgc->nr_of_cells )
@@ -612,7 +628,7 @@ static int DemuxBlock( demux_t *p_demux, const uint8_t *p, int len )
             int i_mux_rate;
             if( !ps_pkt_parse_pack( p_pkt, &i_scr, &i_mux_rate ) )
             {
-                es_out_SetPCR( p_demux->out, i_scr );
+                es_out_SetPCR( p_demux->out, VLC_TS_0 + i_scr );
                 if( i_mux_rate > 0 ) p_sys->i_mux_rate = i_mux_rate;
             }
             block_Release( p_pkt );
@@ -797,8 +813,12 @@ static int DvdReadSetArea( demux_t *p_demux, int i_title, int i_chapter,
         p_sys->i_title_blocks = 0;
         for( int i = i_start_cell; i <= i_end_cell; i++ )
         {
-            p_sys->i_title_blocks += p_pgc->cell_playback[i].last_sector -
-                p_pgc->cell_playback[i].first_sector + 1;
+            const uint32_t cell_blocks = p_pgc->cell_playback[i].last_sector -
+                                         p_pgc->cell_playback[i].first_sector + 1;
+            if(unlikely( cell_blocks == 0 || cell_blocks > INT_MAX ||
+                 INT_MAX - p_sys->i_title_blocks < (int)cell_blocks ))
+                return VLC_EGENERIC;
+            p_sys->i_title_blocks += cell_blocks;
         }
 
         msg_Dbg( p_demux, "title %d vts_title %d pgc %d pgn %d "
@@ -1069,13 +1089,11 @@ static int DvdReadSetArea( demux_t *p_demux, int i_title, int i_chapter,
  * This one is used by the input and translate chronological position from
  * input to logical position on the device.
  *****************************************************************************/
-static void DvdReadSeek( demux_t *p_demux, int i_block_offset )
+static int DvdReadSeek( demux_t *p_demux, int i_block_offset )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
     int i_chapter = 0;
     int i_cell = 0;
-    int i_vobu = 0;
-    int i_sub_cell = 0;
     int i_block;
 
 #define p_pgc p_sys->p_cur_pgc
@@ -1095,7 +1113,7 @@ static void DvdReadSeek( demux_t *p_demux, int i_block_offset )
     if( i_cell > p_sys->i_title_end_cell )
     {
         msg_Err( p_demux, "couldn't find cell for block %i", i_block_offset );
-        return;
+        return VLC_EGENERIC;
     }
     i_block += p_pgc->cell_playback[i_cell].first_sector;
     p_sys->i_title_offset = i_block_offset;
@@ -1123,19 +1141,30 @@ static void DvdReadSeek( demux_t *p_demux, int i_block_offset )
     }
 
     /* Find vobu */
-    while( (int)p_vts->vts_vobu_admap->vobu_start_sectors[i_vobu] <= i_block )
+    /* see ifo_read.c / ifoRead_VOBU_ADMAP_internal for index count */
+    int i_vobu = 1;
+    const size_t i_vobu_sect_index_count =
+            (p_vts->vts_vobu_admap->last_byte + 1 - VOBU_ADMAP_SIZE) / sizeof(uint32_t);
+    for( size_t i=0; i<i_vobu_sect_index_count; i++ )
     {
-        i_vobu++;
-    }
-
-    /* Find sub_cell */
-    while( p_vts->vts_c_adt->cell_adr_table[i_sub_cell].start_sector <
-           p_vts->vts_vobu_admap->vobu_start_sectors[i_vobu-1] )
-    {
-        i_sub_cell++;
+        if( p_vts->vts_vobu_admap->vobu_start_sectors[i] > (uint32_t) i_block )
+            break;
+        i_vobu = i + 1;
     }
 
 #if 1
+    int i_sub_cell = 1;
+    /* Find sub_cell */
+    /* need to check cell # <= vob count as cell table alloc only ensures:
+     * info_length / sizeof(cell_adr_t) < c_adt->nr_of_vobs, see ifo_read.c */
+    const uint32_t vobu_start_sector = p_vts->vts_vobu_admap->vobu_start_sectors[i_vobu-1];
+    for( int i = 0; i + 1<p_vts->vts_c_adt->nr_of_vobs; i++ )
+    {
+        const cell_adr_t *p_cell = &p_vts->vts_c_adt->cell_adr_table[i];
+        if(p_cell->start_sector <= vobu_start_sector)
+           i_sub_cell = i + 1;
+    }
+
     msg_Dbg( p_demux, "cell %d i_sub_cell %d chapter %d vobu %d "
              "cell_sector %d vobu_sector %d sub_cell_sector %d",
              i_cell, i_sub_cell, i_chapter, i_vobu,
@@ -1145,7 +1174,10 @@ static void DvdReadSeek( demux_t *p_demux, int i_block_offset )
 #endif
 
     p_sys->i_cur_block = i_block;
-    p_sys->i_next_vobu = p_vts->vts_vobu_admap->vobu_start_sectors[i_vobu];
+    if(likely( (size_t)i_vobu < i_vobu_sect_index_count ))
+        p_sys->i_next_vobu = p_vts->vts_vobu_admap->vobu_start_sectors[i_vobu];
+    else
+        p_sys->i_next_vobu = i_block;
     p_sys->i_pack_len = p_sys->i_next_vobu - i_block;
     p_sys->i_cur_cell = i_cell;
     p_sys->i_chapter = i_chapter;
@@ -1154,7 +1186,7 @@ static void DvdReadSeek( demux_t *p_demux, int i_block_offset )
 #undef p_vts
 #undef p_pgc
 
-    return;
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
@@ -1164,31 +1196,45 @@ static void DvdReadHandleDSI( demux_t *p_demux, uint8_t *p_data )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
 
-    navRead_DSI( &p_sys->dsi_pack, &p_data[DSI_START_BYTE] );
+    /* Check we are really on a DSI packet
+     * http://www.mpucoder.com/DVD/dsi_pkt.html
+     * Some think it's funny to fill with 0x42 */
+    const uint8_t dsiheader[7] = { 0x00, 0x00, 0x01, 0xbf, 0x03, 0xfa, 0x01 };
+    if(!memcmp(&p_data[DSI_START_BYTE-7], dsiheader, 7))
+    {
+        navRead_DSI( &p_sys->dsi_pack, &p_data[DSI_START_BYTE] );
 
-    /*
-     * Determine where we go next.  These values are the ones we mostly
-     * care about.
-     */
-    p_sys->i_cur_block = p_sys->dsi_pack.dsi_gi.nv_pck_lbn;
-    p_sys->i_pack_len = p_sys->dsi_pack.dsi_gi.vobu_ea;
+        /*
+         * Store the timecodes so we can get the current time
+         */
+        p_sys->i_title_cur_time = (mtime_t) p_sys->dsi_pack.dsi_gi.nv_pck_scr / 90 * 1000;
+        p_sys->i_cell_cur_time = (mtime_t) dvdtime_to_time( &p_sys->dsi_pack.dsi_gi.c_eltm, 0 );
 
-    /*
-     * Store the timecodes so we can get the current time
-     */
-    p_sys->i_title_cur_time = (mtime_t) p_sys->dsi_pack.dsi_gi.nv_pck_scr / 90 * 1000;
-    p_sys->i_cell_cur_time = (mtime_t) dvdtime_to_time( &p_sys->dsi_pack.dsi_gi.c_eltm, 0 );
+        /*
+         * Determine where we go next.  These values are the ones we mostly
+        * care about.
+        */
+        p_sys->i_cur_block = p_sys->dsi_pack.dsi_gi.nv_pck_lbn;
+        p_sys->i_pack_len = p_sys->dsi_pack.dsi_gi.vobu_ea;
 
-    /*
-     * If we're not at the end of this cell, we can determine the next
-     * VOBU to display using the VOBU_SRI information section of the
-     * DSI.  Using this value correctly follows the current angle,
-     * avoiding the doubled scenes in The Matrix, and makes our life
-     * really happy.
-     */
+        /*
+        * If we're not at the end of this cell, we can determine the next
+        * VOBU to display using the VOBU_SRI information section of the
+        * DSI.  Using this value correctly follows the current angle,
+        * avoiding the doubled scenes in The Matrix, and makes our life
+        * really happy.
+        */
 
-    p_sys->i_next_vobu = p_sys->i_cur_block +
-        ( p_sys->dsi_pack.vobu_sri.next_vobu & 0x7fffffff );
+        p_sys->i_next_vobu = p_sys->i_cur_block +
+            ( p_sys->dsi_pack.vobu_sri.next_vobu & 0x7fffffff );
+    }
+    else
+    {
+        /* resync after decoy/corrupted titles */
+        msg_Warn(p_demux, "Invalid DSI packet in VOBU %d found, skipping Cell %d / %d",
+                 p_sys->i_next_vobu, p_sys->i_cur_cell, p_sys->i_title_end_cell);
+        p_sys->dsi_pack.vobu_sri.next_vobu = SRI_END_OF_CELL;
+    }
 
     if( p_sys->dsi_pack.vobu_sri.next_vobu != SRI_END_OF_CELL
         && p_sys->i_angle > 1 )
