@@ -2,7 +2,7 @@
  * cdg.c: CDG decoder module
  *****************************************************************************
  * Copyright (C) 2007 Laurent Aimar
- * $Id: b3427d7a9da3b494313001c0b5d4df3cc93f4fab $
+ * $Id: 084fc3b61b22e7f76a85c5837da5d965efe6faf3 $
  *
  * Authors: Laurent Aimar <fenrir # via.ecp.fr>
  *
@@ -73,7 +73,7 @@ struct decoder_sys_t
 static int  Open ( vlc_object_t * );
 static void Close( vlc_object_t * );
 
-static picture_t *Decode( decoder_t *, block_t ** );
+static int Decode( decoder_t *, block_t * );
 
 static int DecodePacket( decoder_sys_t *p_cdg, uint8_t *p_buffer, int i_buffer );
 static void Flush( decoder_t * );
@@ -86,7 +86,7 @@ vlc_module_begin ()
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_VCODEC )
     set_description( N_("CDG video decoder") )
-    set_capability( "decoder", 1000 )
+    set_capability( "video decoder", 1000 )
     set_callbacks( Open, Close )
     add_shortcut( "cdg" )
 vlc_module_end ()
@@ -113,7 +113,6 @@ static int Open( vlc_object_t *p_this )
 
     /* Set output properties
      * TODO maybe it would be better to use RV16 or RV24 ? */
-    p_dec->fmt_out.i_cat = VIDEO_ES;
     p_dec->fmt_out.i_codec = VLC_CODEC_RGB32;
     p_dec->fmt_out.video.i_width = CDG_DISPLAY_WIDTH;
     p_dec->fmt_out.video.i_height = CDG_DISPLAY_HEIGHT;
@@ -124,8 +123,8 @@ static int Open( vlc_object_t *p_this )
     p_dec->fmt_out.video.i_bmask = 0xff << CDG_COLOR_B_SHIFT;
 
     /* Set callbacks */
-    p_dec->pf_decode_video = Decode;
-    p_dec->pf_flush        = Flush;
+    p_dec->pf_decode = Decode;
+    p_dec->pf_flush  = Flush;
 
     return VLC_SUCCESS;
 }
@@ -145,15 +144,13 @@ static void Flush( decoder_t *p_dec )
  ****************************************************************************
  * This function must be fed with a complete compressed frame.
  ****************************************************************************/
-static picture_t *Decode( decoder_t *p_dec, block_t **pp_block )
+static int Decode( decoder_t *p_dec, block_t *p_block )
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
-    block_t *p_block;
     picture_t *p_pic = NULL;
 
-    if( !pp_block || !*pp_block )
-        return NULL;
-    p_block = *pp_block;
+    if( !p_block ) /* No Drain */
+        return VLCDEC_SUCCESS;
 
     if( p_block->i_flags & BLOCK_FLAG_CORRUPTED )
     {
@@ -173,6 +170,8 @@ static picture_t *Decode( decoder_t *p_dec, block_t **pp_block )
     if( (p_sys->i_packet%3) == 1 && p_block->i_pts == p_block->i_dts )
     {
         /* Get a new picture */
+        if( decoder_UpdateVideoFormat( p_dec ) )
+            goto exit;
         p_pic = decoder_NewPicture( p_dec );
         if( !p_pic )
             goto exit;
@@ -183,8 +182,9 @@ static picture_t *Decode( decoder_t *p_dec, block_t **pp_block )
 
 exit:
     block_Release( p_block );
-    *pp_block = NULL;
-    return p_pic;
+    if( p_pic != NULL )
+        decoder_QueueVideo( p_dec, p_pic );
+    return VLCDEC_SUCCESS;
 }
 
 /*****************************************************************************
