@@ -71,7 +71,7 @@ static unsigned int pi_channels_maps[7] =
 static int  OpenDecoder   ( vlc_object_t * );
 static void CloseDecoder  ( vlc_object_t * );
 
-static block_t *DecodeFrame( decoder_t *, block_t ** );
+static int DecodeFrame( decoder_t *, block_t * );
 static void Flush( decoder_t * );
 
 /*****************************************************************************
@@ -80,8 +80,8 @@ static void Flush( decoder_t * );
 vlc_module_begin();
     set_category( CAT_INPUT );
     set_subcategory( SUBCAT_INPUT_ACODEC );
-    set_description( _("WMA v1/v2 fixed point audio decoder") );
-    set_capability( "decoder", 80 );
+    set_description( N_("WMA v1/v2 fixed point audio decoder") );
+    set_capability( "audio decoder", 80 );
     add_shortcut( "wmafixed" )
     set_callbacks( OpenDecoder, CloseDecoder );
 vlc_module_end();
@@ -98,6 +98,8 @@ static block_t *SplitBuffer( decoder_t *p_dec )
 
     if( i_samples == 0 ) return NULL;
 
+    if( decoder_UpdateAudioFormat( p_dec ) )
+        return NULL;
     if( !( p_buffer = decoder_NewAudioBuffer( p_dec, i_samples ) ) )
         return NULL;
 
@@ -137,7 +139,6 @@ static int OpenDecoder( vlc_object_t *p_this )
     date_Init( &p_sys->end_date, p_dec->fmt_in.audio.i_rate, 1 );
 
     /* Set output properties */
-    p_dec->fmt_out.i_cat = AUDIO_ES;
     p_dec->fmt_out.i_codec = VLC_CODEC_S32N;
     p_dec->fmt_out.audio.i_bitspersample = p_dec->fmt_in.audio.i_bitspersample;
     p_dec->fmt_out.audio.i_rate = p_dec->fmt_in.audio.i_rate;
@@ -147,9 +148,8 @@ static int OpenDecoder( vlc_object_t *p_this )
     assert( p_dec->fmt_out.audio.i_channels <
         ( sizeof( pi_channels_maps ) / sizeof( pi_channels_maps[0] ) ) );
 
-    p_dec->fmt_out.audio.i_original_channels =
-        p_dec->fmt_out.audio.i_physical_channels =
-            pi_channels_maps[p_dec->fmt_out.audio.i_channels];
+    p_dec->fmt_out.audio.i_physical_channels =
+        pi_channels_maps[p_dec->fmt_out.audio.i_channels];
 
     /* aout core assumes this number is not 0 and uses it in divisions */
     assert( p_dec->fmt_out.audio.i_physical_channels != 0 );
@@ -184,8 +184,8 @@ static int OpenDecoder( vlc_object_t *p_this )
     }
 
     /* Set callback */
-    p_dec->pf_decode_audio = DecodeFrame;
-    p_dec->pf_flush        = Flush;
+    p_dec->pf_decode = DecodeFrame;
+    p_dec->pf_flush  = Flush;
 
     return VLC_SUCCESS;
 }
@@ -203,15 +203,13 @@ static void Flush( decoder_t *p_dec )
 /*****************************************************************************
  * DecodeFrame: decodes a wma frame.
  *****************************************************************************/
-static block_t *DecodeFrame( decoder_t *p_dec, block_t **pp_block )
+static int DecodeFrame( decoder_t *p_dec, block_t *p_block )
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
-    block_t       *p_block;
     block_t       *p_aout_buffer = NULL;
 
-    if( !pp_block || !*pp_block ) return NULL;
-
-    p_block = *pp_block;
+    if( p_block == NULL ) /* No Drain */
+        return VLCDEC_SUCCESS;
 
     if( p_block->i_flags & (BLOCK_FLAG_DISCONTINUITY|BLOCK_FLAG_CORRUPTED) )
     {
@@ -219,26 +217,10 @@ static block_t *DecodeFrame( decoder_t *p_dec, block_t **pp_block )
         if( p_block->i_flags & BLOCK_FLAG_CORRUPTED)
         {
             block_Release( p_block );
-            *pp_block = NULL;
-            return NULL;
+            return VLCDEC_SUCCESS;
         }
     }
 
-    if( p_block->i_buffer <= 0 )
-    {
-        /* we already decoded the samples, just feed a few to aout */
-        if( p_sys->i_samples )
-            p_aout_buffer = SplitBuffer( p_dec );
-        if( !p_sys->i_samples )
-        {   /* we need to decode new samples now */
-            free( p_sys->p_output );
-            p_sys->p_output = NULL;
-            block_Release( p_block );
-            *pp_block = NULL;
-        }
-        return p_aout_buffer;
-    }
-
     /* Date management */
     if( p_block->i_pts > VLC_TS_INVALID &&
         p_block->i_pts != date_Get( &p_sys->end_date ) )
@@ -251,7 +233,7 @@ static block_t *DecodeFrame( decoder_t *p_dec, block_t **pp_block )
     {
         /* We've just started the stream, wait for the first PTS. */
         block_Release( p_block );
-        return NULL;
+        return VLCDEC_SUCCESS;
     }
 
     if( wma_decode_superframe_init( &p_sys->wmadec, p_block->p_buffer,
@@ -259,29 +241,27 @@ static block_t *DecodeFrame( decoder_t *p_dec, block_t **pp_block )
     {
         msg_Err( p_dec, "failed initializing wmafixed decoder" );
         block_Release( p_block );
-        *pp_block = NULL;
-        return NULL;
+        return VLCDEC_SUCCESS;
     }
 
     if( p_sys->wmadec.nb_frames <= 0 )
     {
         msg_Err( p_dec, "can not decode, invalid ASF packet ?" );
         block_Release( p_block );
-        *pp_block = NULL;
-        return NULL;
+        return VLCDEC_SUCCESS;
     }
 
     /* worst case */
     size_t i_buffer = BLOCK_MAX_SIZE * MAX_CHANNELS * p_sys->wmadec.nb_frames;
     free( p_sys->p_output );
-    p_sys->p_output = malloc(i_buffer * sizeof(int32_t) );
+    p_sys->p_output = vlc_alloc(i_buffer, sizeof(int32_t));
     p_sys->p_samples = (int8_t*)p_sys->p_output;
 
     if( !p_sys->p_output )
     {
         /* OOM, will try a bit later if VLC hasn't been killed */
         block_Release( p_block );
-        return NULL;
+        return VLCDEC_SUCCESS;
     }
 
     p_sys->i_samples = 0;
@@ -300,20 +280,26 @@ static block_t *DecodeFrame( decoder_t *p_dec, block_t **pp_block )
                 "wma_decode_superframe_frame() failed for frame %d", i );
             free( p_sys->p_output );
             p_sys->p_output = NULL;
-            return NULL;
+            return VLCDEC_SUCCESS;
         }
         p_sys->i_samples += i_samples; /* advance in the samples buffer */
     }
 
-    p_block->i_buffer = 0; /* this block has been decoded */
+    block_Release( p_block ); /* this block has been decoded */
 
     for( size_t s = 0 ; s < i_buffer; s++ )
         p_sys->p_output[s] <<= 2; /* Q30 -> Q32 translation */
 
-    p_aout_buffer = SplitBuffer( p_dec );
-    assert( p_aout_buffer );
+    while( ( p_aout_buffer = SplitBuffer( p_dec ) ) != NULL )
+        decoder_QueueAudio( p_dec, p_aout_buffer );
 
-    return p_aout_buffer;
+    if( !p_sys->i_samples )
+    {   /* we need to decode new samples now */
+        free( p_sys->p_output );
+        p_sys->p_output = NULL;
+    }
+
+    return VLCDEC_SUCCESS;
 }
 
 /*****************************************************************************
