@@ -2,7 +2,7 @@
  * motion.c: control VLC with laptop built-in motion sensors
  *****************************************************************************
  * Copyright (C) 2006 - 2007 the VideoLAN team
- * $Id: 6c2144b5c967ea3f767c4befb1b99ec3d7353865 $
+ * $Id: 7491cadb9c81992272a044ae23357ccba52665c9 $
  *
  * Author: Sam Hocevar <sam@zoy.org>
  *         Jérôme Decoodt <djc@videolan.org> (unimotion integration)
@@ -80,7 +80,7 @@ vlc_module_end ()
 /*****************************************************************************
  * OpenIntf: initialise interface
  *****************************************************************************/
-int Open ( vlc_object_t *p_this )
+static int Open ( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
     intf_sys_t *p_sys = malloc( sizeof( *p_sys ) );
@@ -109,7 +109,7 @@ error:
 /*****************************************************************************
  * CloseIntf: destroy interface
  *****************************************************************************/
-void Close ( vlc_object_t *p_this )
+static void Close ( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
     intf_sys_t *p_sys = p_intf->p_sys;
