@@ -22,7 +22,7 @@
 
 #include <vlc_common.h>
 #include "StreamFormat.hpp"
-#include "ChunksSource.hpp"
+#include "AbstractSource.hpp"
 #include "SegmentTracker.hpp"
 
 #include "plumbing/CommandsQueue.hpp"
@@ -49,7 +49,7 @@ namespace adaptive
     using namespace http;
     using namespace playlist;
 
-    class AbstractStream : public ChunksSource,
+    class AbstractStream : public AbstractSource,
                            public ExtraFMTInfoInterface,
                            public SegmentTrackerListenerInterface,
                            public DemuxerFactoryInterface
@@ -66,10 +66,9 @@ namespace adaptive
         mtime_t getFirstDTS() const;
         int esCount() const;
         bool isSelected() const;
-        bool canActivate() const;
         virtual bool reactivate(mtime_t);
-        void setDisabled(bool);
         bool isDisabled() const;
+        bool isValid() const;
         typedef enum {
             status_eof = 0, /* prioritized */
             status_discontinuity,
@@ -89,7 +88,8 @@ namespace adaptive
         status dequeue(mtime_t, mtime_t *);
         bool decodersDrained();
         virtual bool setPosition(mtime_t, bool);
-        mtime_t getPlaybackTime() const;
+        bool getMediaPlaybackTimes(mtime_t *, mtime_t *, mtime_t *,
+                                   mtime_t *, mtime_t *) const;
         void runUpdates();
 
         /* Used by demuxers fake streams */
@@ -102,10 +102,10 @@ namespace adaptive
 
     protected:
         bool seekAble() const;
-        virtual void setTimeOffset(mtime_t);
+        void setDisabled(bool);
         virtual block_t *checkBlock(block_t *, bool) = 0;
         AbstractDemuxer * createDemux(const StreamFormat &);
-        virtual AbstractDemuxer * newDemux(demux_t *, const StreamFormat &,
+        virtual AbstractDemuxer * newDemux(vlc_object_t *, const StreamFormat &,
                                            es_out_t *, AbstractSourceStream *) const; /* impl */
         virtual bool startDemux();
         virtual bool restartDemux();
@@ -115,6 +115,7 @@ namespace adaptive
         bool discontinuity;
         bool needrestart;
         bool inrestart;
+        bool demuxfirstchunk;
 
         demux_t *p_realdemux;
         StreamFormat format;
@@ -127,17 +128,20 @@ namespace adaptive
         std::string language;
         std::string description;
 
-        CommandsQueue *commandsqueue;
         AbstractDemuxer *demuxer;
         AbstractSourceStream *demuxersource;
+        FakeESOut::LockedFakeEsOut fakeEsOut();
+        FakeESOut::LockedFakeEsOut fakeEsOut() const;
         FakeESOut *fakeesout; /* to intercept/proxy what is sent from demuxstream */
         vlc_mutex_t lock; /* lock for everything accessed by dequeuing */
 
     private:
-        buffering_status doBufferize(mtime_t, unsigned, unsigned);
+        void declaredCodecs();
+        buffering_status doBufferize(mtime_t, mtime_t, mtime_t);
         buffering_status last_buffer_status;
-        bool dead;
+        bool valid;
         bool disabled;
+        unsigned notfound_sequence;
     };
 
     class AbstractStreamFactory
