@@ -30,6 +30,7 @@
 #include "SegmentTimeline.h"
 #include "AbstractPlaylist.hpp"
 #include "BaseRepresentation.h"
+#include "../encryption/CommonEncryption.hpp"
 
 #include <algorithm>
 #include <cassert>
@@ -58,7 +59,6 @@ void SegmentInformation::init()
     segmentBase = NULL;
     segmentList = NULL;
     mediaSegmentTemplate = NULL;
-    switchpolicy = SWITCH_UNKNOWN;
 }
 
 SegmentInformation::~SegmentInformation()
@@ -162,66 +162,35 @@ std::size_t SegmentInformation::getAllSegments(std::vector<ISegment *> &retSegme
     return retSegments.size();
 }
 
-uint64_t SegmentInformation::getLiveStartSegmentNumber(uint64_t def) const
+bool SegmentInformation::getMediaPlaybackRange(mtime_t *rangeBegin,
+                                               mtime_t *rangeEnd,
+                                               mtime_t *rangeLength) const
 {
-    const mtime_t i_max_buffering = getPlaylist()->getMaxBuffering() +
-                                    /* FIXME: add dynamic pts-delay */ CLOCK_FREQ;
-
-    /* Try to never buffer up to really end */
-    const uint64_t OFFSET_FROM_END = 3;
-
     if( mediaSegmentTemplate )
     {
-        uint64_t start = 0;
-        uint64_t end = 0;
         const Timescale timescale = mediaSegmentTemplate->inheritTimescale();
-
-        SegmentTimeline *timeline = mediaSegmentTemplate->segmentTimeline.Get();
+        const SegmentTimeline *timeline = mediaSegmentTemplate->inheritSegmentTimeline();
         if( timeline )
         {
-            start = timeline->minElementNumber();
-            end = timeline->maxElementNumber();
-            /* Try to never buffer up to really end */
-            end = end - std::min(end - start, OFFSET_FROM_END);
-            stime_t endtime, duration;
-            timeline->getScaledPlaybackTimeDurationBySegmentNumber( end, &endtime, &duration );
+            stime_t startTime, endTime, duration;
+            if(!timeline->getScaledPlaybackTimeDurationBySegmentNumber(timeline->minElementNumber(),
+                                                                       &startTime, &duration) ||
+               !timeline->getScaledPlaybackTimeDurationBySegmentNumber(timeline->maxElementNumber(),
+                                                                       &endTime, &duration))
+                return false;
 
-            if( endtime + duration <= timescale.ToScaled( i_max_buffering ) )
-                return start;
-
-            uint64_t number = timeline->getElementNumberByScaledPlaybackTime(
-                                        endtime + duration - timescale.ToScaled( i_max_buffering ) );
-            if( number < start )
-                number = start;
-            return number;
+            *rangeBegin = timescale.ToTime(startTime);
+            *rangeEnd = timescale.ToTime(endTime+duration);
+            *rangeLength = timescale.ToTime(timeline->getTotalLength());
+            return true;
         }
         /* Else compute, current time and timeshiftdepth based */
         else if( mediaSegmentTemplate->duration.Get() )
         {
-            mtime_t i_delay = getPlaylist()->suggestedPresentationDelay.Get();
-
-            if( i_delay == 0 || i_delay > getPlaylist()->timeShiftBufferDepth.Get() )
-                 i_delay = getPlaylist()->timeShiftBufferDepth.Get();
-
-            if( i_delay < getPlaylist()->getMinBuffering() )
-                i_delay = getPlaylist()->getMinBuffering();
-
-            const uint64_t startnumber = mediaSegmentTemplate->startNumber.Get();
-            end = mediaSegmentTemplate->getCurrentLiveTemplateNumber();
-
-            const uint64_t count = timescale.ToScaled( i_delay ) / mediaSegmentTemplate->duration.Get();
-            if( startnumber + count >= end )
-                start = startnumber;
-            else
-                start = end - count;
-
-            uint64_t bufcount = ( OFFSET_FROM_END + timescale.ToScaled(i_max_buffering) /
-                                  mediaSegmentTemplate->duration.Get() );
-            /* Ensure we always pick > start # of availability window as this segment might no longer be avail */
-            if( end - start <= bufcount )
-                bufcount = end - start - 1;
-
-            return ( end - start > bufcount ) ? end - bufcount : start;
+            *rangeEnd = 0;
+            *rangeBegin = -1 * getPlaylist()->timeShiftBufferDepth.Get();
+            *rangeLength = getPlaylist()->timeShiftBufferDepth.Get();
+            return true;
         }
     }
     else if ( segmentList && !segmentList->getSegments().empty() )
@@ -230,36 +199,33 @@ uint64_t SegmentInformation::getLiveStartSegmentNumber(uint64_t def) const
         const std::vector<ISegment *> list = segmentList->getSegments();
 
         const ISegment *back = list.back();
-        const stime_t bufferingstart = back->startTime.Get() + back->duration.Get() - timescale.ToScaled( i_max_buffering );
-        uint64_t number;
-        if( !segmentList->getSegmentNumberByScaledTime( bufferingstart, &number ) )
-            return list.front()->getSequenceNumber();
-        if( number > list.front()->getSequenceNumber() + OFFSET_FROM_END )
-            number -= OFFSET_FROM_END;
-        else
-            number = list.front()->getSequenceNumber();
-        return number;
+        const stime_t startTime = list.front()->startTime.Get();
+        const stime_t endTime = back->startTime.Get() + back->duration.Get();
+        *rangeBegin = timescale.ToTime(startTime);
+        *rangeEnd = timescale.ToTime(endTime);
+        *rangeLength = timescale.ToTime(segmentList->getTotalLength());
+        return true;
     }
     else if( segmentBase )
     {
         const std::vector<ISegment *> list = segmentBase->subSegments();
-        if(!list.empty())
-            return segmentBase->getSequenceNumber();
+        if(list.empty())
+            return false;
 
         const Timescale timescale = inheritTimescale();
         const ISegment *back = list.back();
-        const stime_t bufferingstart = back->startTime.Get() -
-                (OFFSET_FROM_END * back->duration.Get())- timescale.ToScaled( i_max_buffering );
-        uint64_t number;
-        if( !SegmentInfoCommon::getSegmentNumberByScaledTime( list, bufferingstart, &number ) )
-            return list.front()->getSequenceNumber();
-        return number;
+        const stime_t startTime = list.front()->startTime.Get();
+        const stime_t endTime = back->startTime.Get() + back->duration.Get();
+        *rangeBegin = timescale.ToTime(startTime);
+        *rangeEnd = timescale.ToTime(endTime);
+        *rangeLength = 0;
+        return true;
     }
 
     if(parent)
-        return parent->getLiveStartSegmentNumber(def);
+        return parent->getMediaPlaybackRange(rangeBegin, rangeEnd, rangeLength);
     else
-        return def;
+        return false;
 }
 
 /* Returns wanted segment, or next in sequence if not found */
@@ -283,7 +249,7 @@ ISegment * SegmentInformation::getNextSegment(SegmentInfoType type, uint64_t i_p
             {
                 /* Check if we don't exceed timeline */
                 MediaSegmentTemplate *templ = dynamic_cast<MediaSegmentTemplate*>(retSegments[0]);
-                SegmentTimeline *timeline = (templ) ? templ->segmentTimeline.Get() : NULL;
+                const SegmentTimeline *timeline = (templ) ? templ->inheritSegmentTimeline() : NULL;
                 if(timeline)
                 {
                     *pi_newpos = std::max(timeline->minElementNumber(), i_pos);
@@ -294,7 +260,7 @@ ISegment * SegmentInformation::getNextSegment(SegmentInfoType type, uint64_t i_p
                 {
                     *pi_newpos = i_pos;
                     /* start number */
-                    *pi_newpos = std::max((uint64_t)templ->startNumber.Get(), i_pos);
+                    *pi_newpos = std::max(templ->inheritStartNumber(), i_pos);
                 }
                 return seg;
             }
@@ -319,8 +285,8 @@ ISegment * SegmentInformation::getSegment(SegmentInfoType type, uint64_t pos) co
         if(size == 1 && retSegments[0]->isTemplate())
         {
             MediaSegmentTemplate *templ = dynamic_cast<MediaSegmentTemplate*>(retSegments[0]);
-            if(!templ || templ->segmentTimeline.Get() == NULL ||
-               templ->segmentTimeline.Get()->maxElementNumber() > pos)
+            const SegmentTimeline *tl = templ->inheritSegmentTimeline();
+            if(!templ || tl == NULL || tl->maxElementNumber() > pos)
                 return templ;
         }
         else
@@ -347,13 +313,14 @@ bool SegmentInformation::getSegmentNumberByTime(mtime_t time, uint64_t *ret) con
 {
     if( mediaSegmentTemplate )
     {
-        const Timescale timescale = mediaSegmentTemplate->inheritTimescale();
-
-        SegmentTimeline *timeline = mediaSegmentTemplate->segmentTimeline.Get();
+        const SegmentTimeline *timeline = mediaSegmentTemplate->inheritSegmentTimeline();
         if(timeline)
         {
-            time = timescale.ToScaled(time);
-            *ret = timeline->getElementNumberByScaledPlaybackTime(time);
+            const Timescale timescale = timeline->getTimescale().isValid()
+                                      ? timeline->getTimescale()
+                                      : mediaSegmentTemplate->inheritTimescale();
+            stime_t st = timescale.ToScaled(time);
+            *ret = timeline->getElementNumberByScaledPlaybackTime(st);
             return true;
         }
 
@@ -362,11 +329,20 @@ bool SegmentInformation::getSegmentNumberByTime(mtime_t time, uint64_t *ret) con
         {
             if( getPlaylist()->isLive() )
             {
-                *ret = getLiveStartSegmentNumber( mediaSegmentTemplate->startNumber.Get() );
+                mtime_t now = CLOCK_FREQ * ::time(NULL);
+                if(getPlaylist()->availabilityStartTime.Get())
+                {
+                    if(time >= getPlaylist()->availabilityStartTime.Get() && time < now)
+                        *ret = mediaSegmentTemplate->getLiveTemplateNumber(time, true);
+                    else if(now - getPlaylist()->availabilityStartTime.Get() > time)
+                        *ret = mediaSegmentTemplate->getLiveTemplateNumber(time, false);
+                }
+                else return false;
             }
             else
             {
-                *ret = mediaSegmentTemplate->startNumber.Get();
+                const Timescale timescale = mediaSegmentTemplate->inheritTimescale();
+                *ret = mediaSegmentTemplate->inheritStartNumber();
                 *ret += timescale.ToScaled(time) / duration;
             }
             return true;
@@ -402,17 +378,21 @@ bool SegmentInformation::getPlaybackTimeDurationBySegmentNumber(uint64_t number,
     if( (mediaTemplate = inheritSegmentTemplate()) )
     {
         const Timescale timescale = mediaTemplate->inheritTimescale();
+        const SegmentTimeline * timeline = mediaTemplate->inheritSegmentTimeline();
 
         stime_t stime, sduration;
-        if(mediaTemplate->segmentTimeline.Get())
+        if(timeline)
         {
-            mediaTemplate->segmentTimeline.Get()->
-                getScaledPlaybackTimeDurationBySegmentNumber(number, &stime, &sduration);
+            if(!timeline->getScaledPlaybackTimeDurationBySegmentNumber(number, &stime, &sduration))
+                return false;
         }
         else
         {
-            stime = number * mediaTemplate->duration.Get();
-            sduration = mediaTemplate->duration.Get();
+            uint64_t startNumber = mediaTemplate->inheritStartNumber();
+            if(number < startNumber)
+                return false;
+            sduration = mediaTemplate->inheritDuration();
+            stime = (number - startNumber) * sduration;
         }
         *time = timescale.ToTime(stime);
         *duration = timescale.ToTime(sduration);
@@ -434,6 +414,9 @@ bool SegmentInformation::getPlaybackTimeDurationBySegmentNumber(uint64_t number,
         }
     }
 
+    if(parent)
+        return parent->getPlaybackTimeDurationBySegmentNumber(number, time, duration);
+
     return false;
 }
 
@@ -448,14 +431,14 @@ SegmentInformation * SegmentInformation::getChildByID(const adaptive::ID &id)
     return NULL;
 }
 
-void SegmentInformation::mergeWith(SegmentInformation *updated, mtime_t prunetime)
+void SegmentInformation::updateWith(SegmentInformation *updated)
 {
     /* Support Segment List for now */
     if(segmentList && updated->segmentList)
-        segmentList->mergeWith(updated->segmentList);
+        segmentList->updateWith(updated->segmentList);
 
     if(mediaSegmentTemplate && updated->mediaSegmentTemplate)
-        mediaSegmentTemplate->mergeWith(updated->mediaSegmentTemplate, prunetime);
+        mediaSegmentTemplate->updateWith(updated->mediaSegmentTemplate);
 
     std::vector<SegmentInformation *>::const_iterator it;
     for(it=childs.begin(); it!=childs.end(); ++it)
@@ -463,7 +446,7 @@ void SegmentInformation::mergeWith(SegmentInformation *updated, mtime_t prunetim
         SegmentInformation *child = *it;
         SegmentInformation *updatedChild = updated->getChildByID(child->getID());
         if(updatedChild)
-            child->mergeWith(updatedChild, prunetime);
+            child->updateWith(updatedChild);
     }
     /* FIXME: handle difference */
 }
@@ -473,9 +456,9 @@ void SegmentInformation::mergeWithTimeline(SegmentTimeline *updated)
     MediaSegmentTemplate *templ = inheritSegmentTemplate();
     if(templ)
     {
-        SegmentTimeline *timeline = templ->segmentTimeline.Get();
+        SegmentTimeline *timeline = templ->inheritSegmentTimeline();
         if(timeline)
-            timeline->mergeWith(*updated);
+            timeline->updateWith(*updated);
     }
 }
 
@@ -511,12 +494,16 @@ uint64_t SegmentInformation::translateSegmentNumber(uint64_t num, const SegmentI
     return num;
 }
 
-SegmentInformation::SwitchPolicy SegmentInformation::getSwitchPolicy() const
+const CommonEncryption & SegmentInformation::intheritEncryption() const
 {
-    if(switchpolicy == SWITCH_UNKNOWN)
-        return (parent) ? parent->getSwitchPolicy() : SWITCH_UNAVAILABLE;
-    else
-        return switchpolicy;
+    if(parent && commonEncryption.method == CommonEncryption::Method::NONE)
+        return parent->intheritEncryption();
+    return commonEncryption;
+}
+
+void SegmentInformation::setEncryption(const CommonEncryption &enc)
+{
+    commonEncryption = enc;
 }
 
 mtime_t SegmentInformation::getPeriodStart() const
@@ -527,15 +514,16 @@ mtime_t SegmentInformation::getPeriodStart() const
         return 0;
 }
 
-void SegmentInformation::appendSegmentList(SegmentList *list, bool restamp)
+void SegmentInformation::updateSegmentList(SegmentList *list, bool restamp)
 {
-    if(segmentList)
+    if(segmentList && restamp)
     {
-        segmentList->mergeWith(list, restamp);
+        segmentList->updateWith(list, restamp);
         delete list;
     }
     else
     {
+        delete segmentList;
         segmentList = list;
     }
 }
@@ -551,7 +539,7 @@ void SegmentInformation::setSegmentTemplate(MediaSegmentTemplate *templ)
 {
     if(mediaSegmentTemplate)
     {
-        mediaSegmentTemplate->mergeWith(templ, 0);
+        mediaSegmentTemplate->updateWith(templ);
         delete templ;
     }
     else
@@ -612,11 +600,6 @@ void SegmentInformation::SplitUsingIndex(std::vector<SplitPoint> &splitlist)
     }
 }
 
-void SegmentInformation::setSwitchPolicy(SegmentInformation::SwitchPolicy policy)
-{
-    switchpolicy = policy;
-}
-
 Url SegmentInformation::getUrlSegment() const
 {
     if(baseUrl.Get() && baseUrl.Get()->hasScheme())
@@ -661,3 +644,33 @@ MediaSegmentTemplate * SegmentInformation::inheritSegmentTemplate() const
     else
         return NULL;
 }
+
+void SegmentInformation::setAvailabilityTimeOffset(mtime_t t)
+{
+    availabilityTimeOffset = t;
+}
+
+void SegmentInformation::setAvailabilityTimeComplete(bool b)
+{
+    availabilityTimeComplete = b;
+}
+
+mtime_t SegmentInformation::inheritAvailabilityTimeOffset() const
+{
+    for(const SegmentInformation *p = this; p; p = p->parent)
+    {
+        if(availabilityTimeOffset.isSet())
+            return availabilityTimeOffset.value();
+    }
+    return getPlaylist()->getAvailabilityTimeOffset();
+}
+
+bool SegmentInformation::inheritAvailabilityTimeComplete() const
+{
+    for(const SegmentInformation *p = this; p; p = p->parent)
+    {
+        if(availabilityTimeComplete.isSet())
+            return availabilityTimeComplete.value();
+    }
+    return getPlaylist()->getAvailabilityTimeComplete();
+}
