@@ -22,6 +22,7 @@
 
 #include "ICanonicalUrl.hpp"
 #include "../tools/Properties.hpp"
+#include "../encryption/CommonEncryption.hpp"
 #include "SegmentInfoCommon.h"
 #include <vlc_common.h>
 #include <vector>
@@ -34,6 +35,7 @@ namespace adaptive
         class SegmentList;
         class SegmentTimeline;
         class SegmentTemplate;
+        class MediaSegmentTemplate;
         class AbstractPlaylist;
         class ISegment;
 
@@ -43,18 +45,12 @@ namespace adaptive
                                    public TimescaleAble,
                                    public Unique
         {
+            friend class MediaSegmentTemplate;
             public:
                 SegmentInformation( SegmentInformation * = 0 );
                 explicit SegmentInformation( AbstractPlaylist * );
                 virtual ~SegmentInformation();
-                typedef enum SwitchPolicy
-                {
-                    SWITCH_UNKNOWN,
-                    SWITCH_UNAVAILABLE,
-                    SWITCH_SEGMENT_ALIGNED,
-                    SWITCH_BITSWITCHEABLE
-                } SwitchPolicy;
-                SwitchPolicy getSwitchPolicy() const;
+
                 virtual mtime_t getPeriodStart() const;
                 virtual AbstractPlaylist *getPlaylist() const;
 
@@ -63,6 +59,7 @@ namespace adaptive
                     public:
                         size_t offset;
                         mtime_t time;
+                        mtime_t duration;
                 };
                 void SplitUsingIndex(std::vector<SplitPoint>&);
 
@@ -77,39 +74,45 @@ namespace adaptive
                 ISegment * getSegment(SegmentInfoType, uint64_t = 0) const;
                 ISegment * getNextSegment(SegmentInfoType, uint64_t, uint64_t *, bool *) const;
                 bool getSegmentNumberByTime(mtime_t, uint64_t *) const;
-                mtime_t getPlaybackTimeBySegmentNumber(uint64_t) const;
-                uint64_t getLiveStartSegmentNumber(uint64_t) const;
-                virtual void mergeWith(SegmentInformation *, mtime_t);
+                bool getPlaybackTimeDurationBySegmentNumber(uint64_t, mtime_t *, mtime_t *) const;
+                bool     getMediaPlaybackRange(mtime_t *, mtime_t *, mtime_t *) const;
+                virtual void updateWith(SegmentInformation *);
                 virtual void mergeWithTimeline(SegmentTimeline *); /* ! don't use with global merge */
                 virtual void pruneBySegmentNumber(uint64_t);
                 virtual void pruneByPlaybackTime(mtime_t);
                 virtual uint64_t translateSegmentNumber(uint64_t, const SegmentInformation *) const;
+                void setEncryption(const CommonEncryption &);
+                const CommonEncryption & intheritEncryption() const;
 
             protected:
                 std::size_t getAllSegments(std::vector<ISegment *> &) const;
-                std::size_t getSegments(SegmentInfoType, std::vector<ISegment *>&) const;
+                virtual std::size_t getSegments(SegmentInfoType, std::vector<ISegment *>&) const;
                 std::vector<SegmentInformation *> childs;
                 SegmentInformation * getChildByID( const ID & );
                 SegmentInformation *parent;
-                SwitchPolicy switchpolicy;
 
             public:
-                void setSegmentList(SegmentList *);
+                void updateSegmentList(SegmentList *, bool = false);
                 void setSegmentBase(SegmentBase *);
                 void setSegmentTemplate(MediaSegmentTemplate *);
-                void setSwitchPolicy(SwitchPolicy);
                 virtual Url getUrlSegment() const; /* impl */
                 Property<Url *> baseUrl;
-
-            private:
-                void init();
+                void setAvailabilityTimeOffset(mtime_t);
+                void setAvailabilityTimeComplete(bool);
                 SegmentBase *     inheritSegmentBase() const;
                 SegmentList *     inheritSegmentList() const;
                 MediaSegmentTemplate * inheritSegmentTemplate() const;
+                mtime_t           inheritAvailabilityTimeOffset() const;
+                bool              inheritAvailabilityTimeComplete() const;
 
+            private:
+                void init();
                 SegmentBase     *segmentBase;
                 SegmentList     *segmentList;
                 MediaSegmentTemplate *mediaSegmentTemplate;
+                CommonEncryption commonEncryption;
+                Undef<bool>      availabilityTimeComplete;
+                Undef<mtime_t>   availabilityTimeOffset;
         };
     }
 }
