@@ -72,24 +72,8 @@ static inline int GetValue2b(uint32_t *var, const uint8_t *p, unsigned int *skip
 
 static uint32_t SkipBytes( stream_t *s, uint32_t i_bytes )
 {
-    int i_read;
-    int i_to_read = __MIN(i_bytes, INT_MAX);
-    uint32_t i_bytes_read = 0;
-
-    while( i_bytes )
-    {
-        i_read = stream_Read( s, NULL, i_to_read );
-        i_bytes -= i_read;
-        i_bytes_read += i_read;
-        if ( i_read < i_to_read || i_bytes == 0 )
-        {
-            /* end of stream */
-            return i_bytes_read;
-        }
-        i_to_read = __MIN(i_bytes, INT_MAX);
-    }
-
-    return i_bytes_read;
+    ssize_t i_read = vlc_stream_Read( s, NULL, i_bytes );
+    return i_read > 0 ? (uint32_t) i_read : 0;
 }
 
 static int DemuxSubPayload( asf_packet_sys_t *p_packetsys,
@@ -103,7 +87,7 @@ static int DemuxSubPayload( asf_packet_sys_t *p_packetsys,
         p_packetsys->pf_send( p_packetsys, i_stream_number, pp_frame );
     }
 
-    block_t *p_frag = stream_Block( p_packetsys->p_demux->s, i_sub_payload_data_length );
+    block_t *p_frag = vlc_stream_Block( p_packetsys->p_demux->s, i_sub_payload_data_length );
     if( p_frag == NULL ) {
         msg_Warn( p_packetsys->p_demux, "cannot read data" );
         return -1;
@@ -121,14 +105,15 @@ static int DemuxSubPayload( asf_packet_sys_t *p_packetsys,
 
 static void ParsePayloadExtensions( asf_packet_sys_t *p_packetsys,
                                     const asf_track_info_t *p_tkinfo,
-                                    const asf_packet_t *pkt,
-                                    uint32_t i_length, bool *b_keyframe )
+                                    const uint8_t *p_data, size_t i_data,
+                                    bool *b_keyframe,
+                                    int64_t *pi_extension_pts )
 {
     demux_t *p_demux = p_packetsys->p_demux;
 
-    if ( !p_tkinfo || !p_tkinfo->p_esp || !p_tkinfo->p_esp->p_ext ) return;
-    const uint8_t *p_data = pkt->p_peek + pkt->i_skip + 8;
-    i_length -= 8;
+    if ( !p_tkinfo || !p_tkinfo->p_esp || !p_tkinfo->p_esp->p_ext )
+        return;
+
     uint16_t i_payload_extensions_size;
     asf_payload_extension_system_t *p_ext = NULL;
 
@@ -138,10 +123,10 @@ static void ParsePayloadExtensions( asf_packet_sys_t *p_packetsys,
         p_ext = &p_tkinfo->p_esp->p_ext[i];
         if ( p_ext->i_data_size == 0xFFFF ) /* Variable length extension data */
         {
-            if ( i_length < 2 ) return;
+            if ( i_data < 2 ) return;
             i_payload_extensions_size = GetWLE( p_data );
             p_data += 2;
-            i_length -= 2;
+            i_data -= 2;
             i_payload_extensions_size = 0;
         }
         else
@@ -149,7 +134,7 @@ static void ParsePayloadExtensions( asf_packet_sys_t *p_packetsys,
             i_payload_extensions_size = p_ext->i_data_size;
         }
 
-        if ( i_length < i_payload_extensions_size ) return;
+        if ( i_data < i_payload_extensions_size ) return;
 
         if ( guidcmp( &p_ext->i_extension_id, &mfasf_sampleextension_outputcleanpoint_guid ) )
         {
@@ -178,7 +163,20 @@ static void ParsePayloadExtensions( asf_packet_sys_t *p_packetsys,
             p_packetsys->pf_setaspectratio( p_packetsys, p_tkinfo->p_sp->i_stream_number,
                                             p_data[0], p_data[1] );
         }
-        i_length -= i_payload_extensions_size;
+        else if ( guidcmp( &p_ext->i_extension_id, &asf_dvr_sampleextension_timing_rep_data_guid ) )
+        {
+            if ( i_payload_extensions_size != 48 ) goto sizeerror;
+            const int64_t i_pts = GetQWLE(&p_data[8]);
+            if(i_pts != -1)
+                *pi_extension_pts = i_pts / 10000;
+        }
+#if 0
+        else
+        {
+            msg_Dbg( p_demux, "Unknown extension " GUID_FMT, GUID_PRINT( p_ext->i_extension_id ) );
+        }
+#endif
+        i_data -= i_payload_extensions_size;
         p_data += i_payload_extensions_size;
     }
 
@@ -212,8 +210,9 @@ static int DemuxPayload(asf_packet_sys_t *p_packetsys, asf_packet_t *pkt, int i_
     if (GetValue2b(&i_replicated_data_length, pkt->p_peek, &pkt->i_skip, pkt->left - pkt->i_skip, pkt->property) < 0)
         return -1;
 
-    mtime_t i_base_pts;
-    uint8_t i_pts_delta = 0;
+    int64_t i_pkt_time;
+    uint8_t i_pkt_time_delta = 0;
+    int64_t i_extension_pts = -1;
     uint32_t i_payload_data_length = 0;
     uint32_t i_temp_payload_length = 0;
     *p_packetsys->pi_preroll = __MIN( *p_packetsys->pi_preroll, INT64_MAX );
@@ -228,35 +227,40 @@ static int DemuxPayload(asf_packet_sys_t *p_packetsys, asf_packet_t *pkt, int i_
 
     bool b_ignore_pts = (p_tkinfo->i_cat == VIDEO_ES); /* ignore PTS delta with video when not set by mux */
 
+    if( pkt->left - pkt->i_skip < i_replicated_data_length )
+        return -1;
+
     /* Non compressed */
     if( i_replicated_data_length > 7 ) // should be at least 8 bytes
     {
-        /* Followed by 2 optional DWORDS, offset in media and presentation time */
-        i_base_pts = (mtime_t)GetDWLE( pkt->p_peek + pkt->i_skip + 4 );
+        /* Followed by 2 optional DWORDS, offset in media and *media* presentation time */
+        i_pkt_time = (mtime_t)GetDWLE( pkt->p_peek + pkt->i_skip + 4 );
 
         /* Parsing extensions, See 7.3.1 */
-        ParsePayloadExtensions( p_packetsys, p_tkinfo, pkt,
-                                i_replicated_data_length, &b_packet_keyframe );
-        i_base_pts -= *p_packetsys->pi_preroll;
+        ParsePayloadExtensions( p_packetsys, p_tkinfo,
+                                &pkt->p_peek[pkt->i_skip + 8],
+                                i_replicated_data_length - 8,
+                                &b_packet_keyframe,
+                                &i_extension_pts );
+        i_pkt_time -= *p_packetsys->pi_preroll;
+        if(i_extension_pts != -1)
+            i_extension_pts -= *p_packetsys->pi_preroll;
         pkt->i_skip += i_replicated_data_length;
-
-        if( ! pkt->left || pkt->i_skip >= pkt->left )
-            return -1;
     }
     else if ( i_replicated_data_length == 0 )
     {
         /* optional DWORDS missing */
-        i_base_pts = (mtime_t)pkt->send_time;
+        i_pkt_time = (mtime_t)pkt->send_time;
     }
     /* Compressed payload */
     else if( i_replicated_data_length == 1 )
     {
-        /* i_media_object_offset is presentation time */
-        /* Next byte is Presentation Time Delta */
-        i_pts_delta = pkt->p_peek[pkt->i_skip];
+        /* i_media_object_offset is *media* presentation time */
+        /* Next byte is *media* Presentation Time Delta */
+        i_pkt_time_delta = pkt->p_peek[pkt->i_skip];
         b_ignore_pts = false;
-        i_base_pts = (mtime_t)i_media_object_offset;
-        i_base_pts -= *p_packetsys->pi_preroll;
+        i_pkt_time = (mtime_t)i_media_object_offset;
+        i_pkt_time -= *p_packetsys->pi_preroll;
         pkt->i_skip++;
         i_media_object_offset = 0;
     }
@@ -264,20 +268,31 @@ static int DemuxPayload(asf_packet_sys_t *p_packetsys, asf_packet_t *pkt, int i_
     {
         /* >1 && <8 Invalid replicated length ! */
         msg_Warn( p_demux, "Invalid replicated data length detected." );
+        if( pkt->length - pkt->i_skip < pkt->padding_length )
+            return -1;
+
         i_payload_data_length = pkt->length - pkt->padding_length - pkt->i_skip;
         goto skip;
     }
 
+    if( ! pkt->left || pkt->i_skip >= pkt->left )
+        return -1;
+
     bool b_preroll_done = ( pkt->send_time > (*p_packetsys->pi_preroll_start/1000 + *p_packetsys->pi_preroll) );
 
-    if (i_base_pts < 0) i_base_pts = 0; // FIXME?
-    i_base_pts *= 1000;
+    if (i_pkt_time < 0) i_pkt_time = 0; // FIXME?
+    i_pkt_time *= 1000;
 
     if( pkt->multiple ) {
         if (GetValue2b(&i_temp_payload_length, pkt->p_peek, &pkt->i_skip, pkt->left - pkt->i_skip, pkt->length_type) < 0)
             return -1;
-    } else
+    }
+    else
+    {
+        if( pkt->length - pkt->i_skip < pkt->padding_length )
+            return -1;
         i_temp_payload_length = pkt->length - pkt->padding_length - pkt->i_skip;
+    }
 
     i_payload_data_length = i_temp_payload_length;
 
@@ -287,7 +302,8 @@ static int DemuxPayload(asf_packet_sys_t *p_packetsys, asf_packet_t *pkt, int i_
               i_payload + 1, i_stream_number, i_media_object_number,
               i_media_object_offset, i_replicated_data_length, i_payload_data_length );
      msg_Dbg( p_demux,
-              "   pts=%"PRId64" st=%"PRIu32, i_base_pts, pkt->send_time );
+              "  extpts=%"PRId64" pkttime=%"PRId64" st=%"PRIu32,
+              (i_extension_pts >= 0) ? i_extension_pts * 1000 : -1, i_pkt_time, pkt->send_time );
 #endif
 
      if( ! i_payload_data_length || i_payload_data_length > pkt->left )
@@ -302,7 +318,7 @@ static int DemuxPayload(asf_packet_sys_t *p_packetsys, asf_packet_t *pkt, int i_
 
     if ( b_preroll_done )
     {
-        mtime_t i_track_time = i_base_pts;
+        mtime_t i_track_time = i_pkt_time;
 
         if ( p_packetsys->pf_updatetime )
             p_packetsys->pf_updatetime( p_packetsys, i_stream_number, i_track_time );
@@ -312,22 +328,35 @@ static int DemuxPayload(asf_packet_sys_t *p_packetsys, asf_packet_t *pkt, int i_
         p_packetsys->pf_updatesendtime( p_packetsys, INT64_C(1000) * pkt->send_time );
 
     uint32_t i_subpayload_count = 0;
-    while (i_payload_data_length)
+    while (i_payload_data_length && pkt->i_skip < pkt->left )
     {
         uint32_t i_sub_payload_data_length = i_payload_data_length;
         if( i_replicated_data_length == 1 )
         {
             i_sub_payload_data_length = pkt->p_peek[pkt->i_skip++];
             i_payload_data_length--;
+            if( i_sub_payload_data_length > i_payload_data_length )
+                goto skip;
         }
 
         SkipBytes( p_demux->s, pkt->i_skip );
 
-        mtime_t i_payload_pts = i_base_pts + (mtime_t)i_pts_delta * i_subpayload_count * 1000;
-        if ( p_tkinfo->p_sp )
-            i_payload_pts -= p_tkinfo->p_sp->i_time_offset * 10;
+        mtime_t i_payload_pts;
+#if 0
+        if( i_extension_pts != -1 )
+        {
+            i_payload_pts = i_extension_pts * 1000;
+            b_ignore_pts = false;
+        }
+        else
+#endif
+        {
+            i_payload_pts = i_pkt_time + (mtime_t)i_pkt_time_delta * i_subpayload_count * 1000;
+            if ( p_tkinfo->p_sp )
+                i_payload_pts -= p_tkinfo->p_sp->i_time_offset * 10;
+        }
 
-        mtime_t i_payload_dts = i_base_pts;
+        mtime_t i_payload_dts = i_pkt_time;
 
         if ( p_tkinfo->p_sp )
             i_payload_dts -= p_tkinfo->p_sp->i_time_offset * 10;
@@ -345,8 +374,8 @@ static int DemuxPayload(asf_packet_sys_t *p_packetsys, asf_packet_t *pkt, int i_
         pkt->i_skip = 0;
         if( pkt->left > 0 )
         {
-            int i_return = stream_Peek( p_demux->s, &pkt->p_peek, __MIN(pkt->left, INT_MAX) );
-            if ( i_return <= 0 || (unsigned int) i_return < __MIN(pkt->left, INT_MAX) )
+            ssize_t i_return = vlc_stream_Peek( p_demux->s, &pkt->p_peek, pkt->left );
+            if ( i_return <= 0 || (size_t) i_return < pkt->left )
             {
             msg_Warn( p_demux, "cannot peek, EOF ?" );
             return -1;
@@ -374,8 +403,8 @@ int DemuxASFPacket( asf_packet_sys_t *p_packetsys,
     demux_t *p_demux = p_packetsys->p_demux;
 
     const uint8_t *p_peek;
-    int i_return = stream_Peek( p_demux->s, &p_peek,i_data_packet_min );
-    if( i_return <= 0 || ((unsigned int) i_return) < i_data_packet_min )
+    ssize_t i_return = vlc_stream_Peek( p_demux->s, &p_peek,i_data_packet_min );
+    if( i_return <= 0 || (size_t) i_return < i_data_packet_min )
     {
         msg_Warn( p_demux, "cannot peek while getting new packet, EOF ?" );
         return 0;
@@ -435,11 +464,14 @@ int DemuxASFPacket( asf_packet_sys_t *p_packetsys,
         pkt.length = i_data_packet_min;
     }
 
+    if( i_skip + 4 > i_data_packet_min )
+        goto loop_error_recovery;
+
     pkt.send_time = GetDWLE( p_peek + i_skip ); i_skip += 4;
     /* uint16_t i_packet_duration = GetWLE( p_peek + i_skip ); */ i_skip += 2;
 
-    i_return = stream_Peek( p_demux->s, &p_peek, pkt.length );
-    if( i_return <= 0 || pkt.length == 0 || (unsigned int)i_return < pkt.length )
+    i_return = vlc_stream_Peek( p_demux->s, &p_peek, pkt.length );
+    if( i_return <= 0 || pkt.length == 0 || (size_t)i_return < pkt.length )
     {
         msg_Warn( p_demux, "cannot peek, EOF ?" );
         return 0;
@@ -449,6 +481,8 @@ int DemuxASFPacket( asf_packet_sys_t *p_packetsys,
     pkt.length_type = 0x02; //unused
     if( pkt.multiple )
     {
+        if( i_skip + 1 >= i_data_packet_min )
+            goto loop_error_recovery;
         i_payload_count = p_peek[i_skip] & 0x3f;
         pkt.length_type = ( p_peek[i_skip] >> 6 )&0x03;
         i_skip++;
@@ -479,8 +513,8 @@ int DemuxASFPacket( asf_packet_sys_t *p_packetsys,
             msg_Warn( p_demux, "Read %"PRIu32" too much bytes in the packet",
                             pkt.padding_length - pkt.left );
 #endif
-        int i_return = stream_Read( p_demux->s, NULL, pkt.left );
-        if( i_return < 0 || (unsigned int) i_return < pkt.left )
+        i_return = vlc_stream_Read( p_demux->s, NULL, pkt.left );
+        if( i_return < 0 || (size_t) i_return < pkt.left )
         {
             msg_Err( p_demux, "cannot skip data, EOF ?" );
             return 0;
@@ -496,8 +530,8 @@ loop_error_recovery:
         msg_Err( p_demux, "unsupported packet header, fatal error" );
         return -1;
     }
-    i_return = stream_Read( p_demux->s, NULL, i_data_packet_min );
-    if( i_return <= 0 || (unsigned int) i_return != i_data_packet_min )
+    i_return = vlc_stream_Read( p_demux->s, NULL, i_data_packet_min );
+    if( i_return <= 0 || (size_t) i_return != i_data_packet_min )
     {
         msg_Warn( p_demux, "cannot skip data, EOF ?" );
         return 0;
