@@ -33,7 +33,7 @@ namespace dash
 
         protected:
             virtual block_t *checkBlock(block_t *, bool); /* impl */
-            virtual AbstractDemuxer * newDemux(demux_t *, const StreamFormat &,
+            virtual AbstractDemuxer * newDemux(vlc_object_t *, const StreamFormat &,
                                                es_out_t *, AbstractSourceStream *) const; /* reimpl */
     };
 
