@@ -23,11 +23,8 @@
 
 #include "M3U8.hpp"
 #include "Representation.hpp"
-#include "../adaptive/playlist/BasePeriod.h"
-#include "../adaptive/playlist/BaseAdaptationSet.h"
-
-#include <vlc_common.h>
-#include <vlc_stream.h>
+#include "../../adaptive/playlist/BasePeriod.h"
+#include "../../adaptive/playlist/BaseAdaptationSet.h"
 
 using namespace hls::playlist;
 
@@ -39,11 +36,11 @@ M3U8::M3U8 (vlc_object_t *p_object) :
 
 M3U8::~M3U8()
 {
-
 }
 
 bool M3U8::isLive() const
 {
+    bool b_live = false;
     std::vector<BasePeriod *>::const_iterator itp;
     for(itp = periods.begin(); itp != periods.end(); ++itp)
     {
@@ -56,13 +53,18 @@ bool M3U8::isLive() const
             for(itr = adaptSet->getRepresentations().begin(); itr != adaptSet->getRepresentations().end(); ++itr)
             {
                 const Representation *rep = dynamic_cast<const Representation *>(*itr);
-                if(rep->initialized() && rep->isLive())
-                    return true;
+                if(rep->initialized())
+                {
+                    if(rep->isLive())
+                        b_live = true;
+                    else
+                        return false; /* Any non live has higher priority */
+                }
             }
         }
     }
 
-    return false;
+    return b_live;
 }
 
 void M3U8::debug()
