@@ -2,7 +2,7 @@
  * chapter_command.cpp : matroska demuxer
  *****************************************************************************
  * Copyright (C) 2003-2004 VLC authors and VideoLAN
- * $Id: 36d728012f8b5e3d1e22b534d01a8c83766fd1ac $
+ * $Id: 1ec3ffe9f26c5cf51c6de8908561a1c7c1e5ab36 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Steve Lhomme <steve.lhomme@free.fr>
@@ -23,42 +23,32 @@
  *****************************************************************************/
 
 #include "chapter_command.hpp"
+#include <algorithm>
 
 void chapter_codec_cmds_c::AddCommand( const KaxChapterProcessCommand & command )
 {
     uint32 codec_time = uint32(-1);
     for( size_t i = 0; i < command.ListSize(); i++ )
     {
-        const EbmlElement *k = command[i];
-
-        if( MKV_IS_ID( k, KaxChapterProcessTime ) )
+        if( MKV_CHECKED_PTR_DECL( p_cpt, KaxChapterProcessTime const, command[i] ) )
         {
-            codec_time = uint32( *static_cast<const KaxChapterProcessTime*>( k ) );
+            codec_time = static_cast<uint32>( *p_cpt );
             break;
         }
     }
 
     for( size_t i = 0; i < command.ListSize(); i++ )
     {
-        const EbmlElement *k = command[i];
-
-        if( MKV_IS_ID( k, KaxChapterProcessData ) )
+        if( MKV_CHECKED_PTR_DECL( p_cpd, KaxChapterProcessData const, command[i] ) )
         {
-            KaxChapterProcessData *p_data =  new KaxChapterProcessData( *static_cast<const KaxChapterProcessData*>( k ) );
-            switch ( codec_time )
-            {
-            case 0:
-                during_cmds.push_back( p_data );
-                break;
-            case 1:
-                enter_cmds.push_back( p_data );
-                break;
-            case 2:
-                leave_cmds.push_back( p_data );
-                break;
-            default:
-                delete p_data;
-            }
+            std::vector<KaxChapterProcessData*> *containers[] = {
+                &during_cmds, /* codec_time = 0 */
+                &enter_cmds,  /* codec_time = 1 */
+                &leave_cmds   /* codec_time = 2 */
+            };
+
+            if( codec_time < 3 )
+                containers[codec_time]->push_back( new KaxChapterProcessData( *p_cpd ) );
         }
     }
 }
@@ -78,50 +68,36 @@ int16 dvd_chapter_codec_c::GetTitleNumber()
 
 bool dvd_chapter_codec_c::Enter()
 {
-    bool f_result = false;
-    std::vector<KaxChapterProcessData*>::iterator index = enter_cmds.begin();
-    while ( index != enter_cmds.end() )
-    {
-        if ( (*index)->GetSize() )
-        {
-            binary *p_data = (*index)->GetBuffer();
-            size_t i_size = *p_data++;
-            // avoid reading too much from the buffer
-            i_size = __MIN( i_size, ((*index)->GetSize() - 1) >> 3 );
-            for ( ; i_size > 0; i_size--, p_data += 8 )
-            {
-                msg_Dbg( &sys.demuxer, "Matroska DVD enter command" );
-                f_result |= sys.dvd_interpretor.Interpret( p_data );
-            }
-        }
-        ++index;
-    }
-    return f_result;
+    return EnterLeaveHelper( "Matroska DVD enter command", &enter_cmds );
 }
 
 bool dvd_chapter_codec_c::Leave()
+{
+    return EnterLeaveHelper( "Matroska DVD leave command", &leave_cmds );
+}
+
+bool dvd_chapter_codec_c::EnterLeaveHelper( char const * str_diag, std::vector<KaxChapterProcessData*> * p_container )
 {
     bool f_result = false;
-    std::vector<KaxChapterProcessData*>::iterator index = leave_cmds.begin();
-    while ( index != leave_cmds.end() )
+    std::vector<KaxChapterProcessData*>::iterator it = p_container->begin ();
+    while( it != p_container->end() )
     {
-        if ( (*index)->GetSize() )
+        if( (*it)->GetSize() )
         {
-            binary *p_data = (*index)->GetBuffer();
-            size_t i_size = *p_data++;
-            // avoid reading too much from the buffer
-            i_size = __MIN( i_size, ((*index)->GetSize() - 1) >> 3 );
-            for ( ; i_size > 0; i_size--, p_data += 8 )
+            binary *p_data = (*it)->GetBuffer();
+            size_t i_size  = std::min<size_t>( *p_data++, ( (*it)->GetSize() - 1 ) >> 3 ); // avoid reading too much
+            for( ; i_size > 0; i_size -=1, p_data += 8 )
             {
-                msg_Dbg( &sys.demuxer, "Matroska DVD leave command" );
+                msg_Dbg( &sys.demuxer, "%s", str_diag);
                 f_result |= sys.dvd_interpretor.Interpret( p_data );
             }
         }
-        ++index;
+        ++it;
     }
     return f_result;
 }
 
+
 std::string dvd_chapter_codec_c::GetCodecName( bool f_for_title ) const
 {
     std::string result;
@@ -171,8 +147,8 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
     if ( i_size != 8 )
         return false;
 
-    virtual_segment_c *p_segment = NULL;
-    virtual_chapter_c *p_chapter = NULL;
+    virtual_segment_c *p_vsegment = NULL;
+    virtual_chapter_c *p_vchapter = NULL;
     bool f_result = false;
     uint16 i_command = ( p_command[0] << 8 ) + p_command[1];
 
@@ -204,12 +180,12 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
         case 7:
             if ( ((p_command[1] >> 4) & 0x7) == 0)
             {
-                i_cr1 = p_command[2];
+                i_cr1 = p_command[4];
                 i_cr2 = (p_command[6] << 8) + p_command[7];
             }
             else
             {
-                i_cr1 = p_command[2];
+                i_cr1 = p_command[5];
                 i_cr2 = (p_command[6] << 8) + p_command[7];
             }
             break;
@@ -307,11 +283,17 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
             msg_Dbg( &sys.demuxer, "JumpTT %d", i_title );
 
             // find in the ChapProcessPrivate matching this Title level
-            p_chapter = sys.BrowseCodecPrivate( 1, MatchTitleNumber, &i_title, sizeof(i_title), p_segment );
-            if ( p_segment != NULL )
+            p_vchapter = sys.BrowseCodecPrivate( 1, MatchTitleNumber, &i_title, sizeof(i_title), p_vsegment );
+            if ( p_vsegment != NULL && p_vchapter != NULL )
             {
-                sys.JumpTo( *p_segment, p_chapter );
-                f_result = true;
+                /* enter via the First Cell */
+                uint8 i_cell = 1;
+                p_vchapter = p_vchapter->BrowseCodecPrivate( MATROSKA_CHAPTER_CODEC_DVD, MatchCellNumber, &i_cell, sizeof(i_cell) );
+                if ( p_vchapter != NULL )
+                {
+                    sys.JumpTo( *p_vsegment, *p_vchapter );
+                    f_result = true;
+                }
             }
 
             break;
@@ -350,11 +332,17 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
                         msg_Dbg( &sys.demuxer, "CallSS <unknown> (rsm_cell %x)", p_command[4]);
                         break;
                     }
-                    p_chapter = sys.BrowseCodecPrivate( 1, MatchPgcType, &p_type, 1, p_segment );
-                    if ( p_segment != NULL )
+                    p_vchapter = sys.BrowseCodecPrivate( 1, MatchPgcType, &p_type, 1, p_vsegment );
+                    if ( p_vsegment != NULL && p_vchapter != NULL )
                     {
-                        sys.JumpTo( *p_segment, p_chapter );
-                        f_result = true;
+                        /* enter via the first Cell */
+                        uint8 i_cell = 1;
+                        p_vchapter = p_vchapter->BrowseCodecPrivate( MATROSKA_CHAPTER_CODEC_DVD, MatchCellNumber, &i_cell, sizeof(i_cell) );
+                        if ( p_vchapter != NULL )
+                        {
+                            sys.JumpTo( *p_vsegment, *p_vchapter );
+                            f_result = true;
+                        }
                     }
                 break;
                 case 1:
@@ -404,13 +392,13 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
                         break;
                     }
                     // find the VMG
-                    p_chapter = sys.BrowseCodecPrivate( 1, MatchIsVMG, NULL, 0, p_segment );
-                    if ( p_segment != NULL )
+                    p_vchapter = sys.BrowseCodecPrivate( 1, MatchIsVMG, NULL, 0, p_vsegment );
+                    if ( p_vsegment != NULL )
                     {
-                        p_chapter = p_segment->BrowseCodecPrivate( 1, MatchPgcType, &p_type, 1 );
-                        if ( p_chapter != NULL )
+                        p_vchapter = p_vsegment->BrowseCodecPrivate( 1, MatchPgcType, &p_type, 1 );
+                        if ( p_vchapter != NULL )
                         {
-                            sys.JumpTo( *p_segment, p_chapter );
+                            sys.JumpTo( *p_vsegment, *p_vchapter );
                             f_result = true;
                         }
                     }
@@ -442,19 +430,19 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
                         break;
                     }
 
-                    p_chapter = sys.BrowseCodecPrivate( 1, MatchVTSMNumber, &p_command[4], 1, p_segment );
+                    p_vchapter = sys.BrowseCodecPrivate( 1, MatchVTSMNumber, &p_command[4], 1, p_vsegment );
 
-                    if ( p_segment != NULL && p_chapter != NULL )
+                    if ( p_vsegment != NULL && p_vchapter != NULL )
                     {
                         // find the title in the VTS
-                        p_chapter = p_chapter->BrowseCodecPrivate( 1, MatchTitleNumber, &p_command[3], 1 );
-                        if ( p_chapter != NULL )
+                        p_vchapter = p_vchapter->BrowseCodecPrivate( 1, MatchTitleNumber, &p_command[3], 1 );
+                        if ( p_vchapter != NULL )
                         {
                             // find the specified menu in the VTSM
-                            p_chapter = p_segment->BrowseCodecPrivate( 1, MatchPgcType, &p_type, 1 );
-                            if ( p_chapter != NULL )
+                            p_vchapter = p_vsegment->BrowseCodecPrivate( 1, MatchPgcType, &p_type, 1 );
+                            if ( p_vchapter != NULL )
                             {
-                                sys.JumpTo( *p_segment, p_chapter );
+                                sys.JumpTo( *p_vsegment, *p_vchapter );
                                 f_result = true;
                             }
                         }
@@ -478,25 +466,25 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
             msg_Dbg( &sys.demuxer, "JumpVTS Title (%d) PTT (%d)", i_title, i_ptt);
 
             // find the current VTS content segment
-            p_chapter = sys.p_current_segment->BrowseCodecPrivate( 1, MatchIsDomain, NULL, 0 );
-            if ( p_chapter != NULL )
+            p_vchapter = sys.p_current_vsegment->BrowseCodecPrivate( 1, MatchIsDomain, NULL, 0 );
+            if ( p_vchapter != NULL )
             {
-                int16 i_curr_title = ( p_chapter->p_chapter )? p_chapter->p_chapter->GetTitleNumber() : 0;
+                int16 i_curr_title = ( p_vchapter->p_chapter )? p_vchapter->p_chapter->GetTitleNumber() : 0;
                 if ( i_curr_title > 0 )
                 {
-                    p_chapter = sys.BrowseCodecPrivate( 1, MatchVTSNumber, &i_curr_title, sizeof(i_curr_title), p_segment );
+                    p_vchapter = sys.BrowseCodecPrivate( 1, MatchVTSNumber, &i_curr_title, sizeof(i_curr_title), p_vsegment );
 
-                    if ( p_segment != NULL && p_chapter != NULL )
+                    if ( p_vsegment != NULL && p_vchapter != NULL )
                     {
                         // find the title in the VTS
-                        p_chapter = p_chapter->BrowseCodecPrivate( 1, MatchTitleNumber, &i_title, sizeof(i_title) );
-                        if ( p_chapter != NULL )
+                        p_vchapter = p_vchapter->BrowseCodecPrivate( 1, MatchTitleNumber, &i_title, sizeof(i_title) );
+                        if ( p_vchapter != NULL )
                         {
                             // find the chapter in the title
-                            p_chapter = p_chapter->BrowseCodecPrivate( 1, MatchChapterNumber, &i_ptt, sizeof(i_ptt) );
-                            if ( p_chapter != NULL )
+                            p_vchapter = p_vchapter->BrowseCodecPrivate( 1, MatchChapterNumber, &i_ptt, sizeof(i_ptt) );
+                            if ( p_vchapter != NULL )
                             {
-                                sys.JumpTo( *p_segment, p_chapter );
+                                sys.JumpTo( *p_vsegment, *p_vchapter );
                                 f_result = true;
                             }
                         }
@@ -526,13 +514,10 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
             uint16 i_pgcn = (p_command[6] << 8) + p_command[7];
 
             msg_Dbg( &sys.demuxer, "Link PGCN(%d)", i_pgcn );
-            p_chapter = sys.p_current_segment->BrowseCodecPrivate( 1, MatchPgcNumber, &i_pgcn, 2 );
-            if ( p_chapter != NULL )
+            p_vchapter = sys.p_current_vsegment->BrowseCodecPrivate( 1, MatchPgcNumber, &i_pgcn, 2 );
+            if ( p_vchapter != NULL )
             {
-                if ( !p_chapter->Enter( true ) )
-                    // jump to the location in the found segment
-                    sys.p_current_segment->Seek( sys.demuxer, p_chapter->i_virtual_start_time, -1, p_chapter, -1 );
-
+                sys.JumpTo( *sys.p_current_vsegment, *p_vchapter );
                 f_result = true;
             }
             break;
@@ -541,16 +526,13 @@ bool dvd_command_interpretor_c::Interpret( const binary * p_command, size_t i_si
         {
             uint8 i_cn = p_command[7];
 
-            p_chapter = sys.p_current_segment->CurrentChapter();
+            p_vchapter = sys.p_current_vsegment->CurrentChapter();
 
             msg_Dbg( &sys.demuxer, "LinkCN (cell %d)", i_cn );
-            p_chapter = p_chapter->BrowseCodecPrivate( 1, MatchCellNumber, &i_cn, 1 );
-            if ( p_chapter != NULL )
+            p_vchapter = p_vchapter->BrowseCodecPrivate( 1, MatchCellNumber, &i_cn, 1 );
+            if ( p_vchapter != NULL )
             {
-                if ( !p_chapter->Enter( true ) )
-                    // jump to the location in the found segment
-                    sys.p_current_segment->Seek( sys.demuxer, p_chapter->i_virtual_start_time, -1, p_chapter, -1 );
-
+                sys.JumpTo( *sys.p_current_vsegment, *p_vchapter );
                 f_result = true;
             }
             break;
@@ -609,7 +591,7 @@ bool dvd_command_interpretor_c::MatchVTSNumber( const chapter_codec_cmds_c &data
         return false;
 
     uint16 i_gtitle = (data.p_private_data->GetBuffer()[2] << 8 ) + data.p_private_data->GetBuffer()[3];
-    uint16 i_title = *(uint16*)p_cookie;
+    uint16 i_title = *static_cast<uint16 const*>( p_cookie );
 
     return (i_gtitle == i_title);
 }
@@ -623,7 +605,7 @@ bool dvd_command_interpretor_c::MatchVTSMNumber( const chapter_codec_cmds_c &dat
         return false;
 
     uint8 i_gtitle = data.p_private_data->GetBuffer()[3];
-    uint8 i_title = *(uint8*)p_cookie;
+    uint8 i_title = *static_cast<uint8 const*>( p_cookie );
 
     return (i_gtitle == i_title);
 }
@@ -637,7 +619,7 @@ bool dvd_command_interpretor_c::MatchTitleNumber( const chapter_codec_cmds_c &da
         return false;
 
     uint16 i_gtitle = (data.p_private_data->GetBuffer()[1] << 8 ) + data.p_private_data->GetBuffer()[2];
-    uint8 i_title = *(uint8*)p_cookie;
+    uint8 i_title = *static_cast<uint8 const*>( p_cookie );
 
     return (i_gtitle == i_title);
 }
@@ -651,7 +633,7 @@ bool dvd_command_interpretor_c::MatchPgcType( const chapter_codec_cmds_c &data,
         return false;
 
     uint8 i_pgc_type = data.p_private_data->GetBuffer()[3] & 0x0F;
-    uint8 i_pgc = *(uint8*)p_cookie;
+    uint8 i_pgc = *static_cast<uint8 const*>( p_cookie );
 
     return (i_pgc_type == i_pgc);
 }
@@ -664,7 +646,7 @@ bool dvd_command_interpretor_c::MatchPgcNumber( const chapter_codec_cmds_c &data
     if ( data.p_private_data->GetBuffer()[0] != MATROSKA_DVD_LEVEL_PGC )
         return false;
 
-    uint16 *i_pgc_n = (uint16 *)p_cookie;
+    uint16 const* i_pgc_n = static_cast<uint16 const*>( p_cookie );
     uint16 i_pgc_num = (data.p_private_data->GetBuffer()[1] << 8) + data.p_private_data->GetBuffer()[2];
 
     return (i_pgc_num == *i_pgc_n);
@@ -679,7 +661,7 @@ bool dvd_command_interpretor_c::MatchChapterNumber( const chapter_codec_cmds_c &
         return false;
 
     uint8 i_chapter = data.p_private_data->GetBuffer()[1];
-    uint8 i_ptt = *(uint8*)p_cookie;
+    uint8 i_ptt = *static_cast<uint8 const*>( p_cookie );
 
     return (i_chapter == i_ptt);
 }
@@ -692,7 +674,7 @@ bool dvd_command_interpretor_c::MatchCellNumber( const chapter_codec_cmds_c &dat
     if ( data.p_private_data->GetBuffer()[0] != MATROSKA_DVD_LEVEL_CN )
         return false;
 
-    uint8 *i_cell_n = (uint8 *)p_cookie;
+    uint8 const* i_cell_n = static_cast<uint8 const*>( p_cookie );
     uint8 i_cell_num = data.p_private_data->GetBuffer()[3];
 
     return (i_cell_num == *i_cell_n);
@@ -706,12 +688,7 @@ bool matroska_script_interpretor_c::Interpret( const binary * p_command, size_t
 {
     bool b_result = false;
 
-    char *psz_str = (char*) malloc( i_size + 1 );
-    memcpy( psz_str, p_command, i_size );
-    psz_str[ i_size ] = '\0';
-
-    std::string sz_command = psz_str;
-    free( psz_str );
+    std::string sz_command( reinterpret_cast<const char*> (p_command), i_size );
 
     msg_Dbg( &sys.demuxer, "command : %s", sz_command.c_str() );
 
@@ -739,17 +716,17 @@ bool matroska_script_interpretor_c::Interpret( const binary * p_command, size_t
         }
 
         std::string st = sz_command.substr( i+1, j-i-1 );
-        int64_t i_chapter_uid = atoi( st.c_str() );
+        int64_t i_chapter_uid = atoll( st.c_str() );
 
-        virtual_segment_c *p_segment;
-        virtual_chapter_c *p_chapter = sys.FindChapter( i_chapter_uid, p_segment );
+        virtual_segment_c *p_vsegment;
+        virtual_chapter_c *p_vchapter = sys.FindChapter( i_chapter_uid, p_vsegment );
 
-        if ( p_chapter == NULL )
+        if ( p_vchapter == NULL )
             msg_Dbg( &sys.demuxer, "Chapter %" PRId64 " not found", i_chapter_uid);
         else
         {
-            if ( !p_chapter->EnterAndLeave( sys.p_current_segment->CurrentChapter() ) )
-                p_segment->Seek( sys.demuxer, p_chapter->i_virtual_start_time, -1, p_chapter, -1 );
+            if ( !p_vchapter->EnterAndLeave( sys.p_current_vsegment->CurrentChapter() ) )
+                p_vsegment->Seek( sys.demuxer, p_vchapter->i_mk_virtual_start_time, p_vchapter );
             b_result = true;
         }
     }
