@@ -1,25 +1,25 @@
 /*****************************************************************************
- * mkv.cpp : matroska demuxer
+ * stream_io_callback.hpp : matroska demuxer
  *****************************************************************************
- * Copyright (C) 2003-2004 the VideoLAN team
- * $Id: 11566daa034958cdd463499093afa16c59185225 $
+ * Copyright (C) 2003-2004 VLC authors and VideoLAN
+ * $Id: d2c72a4214ebbe3d3b9009a4a61acd2d22fcec9f $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Steve Lhomme <steve.lhomme@free.fr>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 #include "mkv.hpp"
 
@@ -34,14 +34,16 @@ class vlc_stream_io_callback: public IOCallback
     bool           b_owner;
 
   public:
-    vlc_stream_io_callback( stream_t *, bool );
+    vlc_stream_io_callback( stream_t *, bool owner );
 
     virtual ~vlc_stream_io_callback()
     {
         if( b_owner )
-            stream_Delete( s );
+            vlc_stream_Delete( s );
     }
 
+    bool IsEOF() const { return mb_eof; }
+
     virtual uint32   read            ( void *p_buffer, size_t i_size);
     virtual void     setFilePointer  ( int64_t i_offset, seek_mode mode = seek_beginning );
     virtual size_t   write           ( const void *p_buffer, size_t i_size);
