@@ -1,24 +1,24 @@
 /*******************************************************************************
- * itml.c : iTunes Music Library import functions
+ * itml.h : iTunes Music Library import functions
  *******************************************************************************
- * Copyright (C) 2007 the VideoLAN team
- * $Id: d352b40fcd7bad2fbcd06c74e8095624269900db $
+ * Copyright (C) 2007 VLC authors and VideoLAN
+ * $Id: e09c8e42888f58fab7a59e6a2c8763dd4f5ef362 $
  *
  * Authors: Yoann Peronneau <yoann@videolan.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *******************************************************************************/
 /**
  * \file modules/demux/playlist/itml.h
@@ -26,19 +26,15 @@
  */
 
 /* defines */
-#define FREE_VALUE()    FREENULL( psz_value )
-#define FREE_KEY()      FREENULL( psz_key )
-#define FREE_ATT()      FREE_VALUE()
-#define FREE_ATT_KEY()  do{ FREE_VALUE();FREE_KEY();} while(0)
-
 #define UNKNOWN_CONTENT 0
 #define SIMPLE_CONTENT 1
 #define COMPLEX_CONTENT 2
 
 #define SIMPLE_INTERFACE  (track_elem_t    *p_track,\
                            const char      *psz_name,\
-                           char            *psz_value)
-#define COMPLEX_INTERFACE (demux_t         *p_demux,\
+                           char            *psz_value,\
+                           void            *opaque)
+#define COMPLEX_INTERFACE (stream_t        *p_demux,\
                            input_item_node_t    *p_input_node,\
                            track_elem_t    *p_track,\
                            xml_reader_t    *p_xml_reader,\
