@@ -28,25 +28,10 @@
 
  */
 
-@interface SideBarItem : NSObject {
-
-NSString *title;
-
-NSString *identifier;
-    NSString *untranslatedTitle;
-
-NSImage *icon;
-
-NSInteger badgeValue;
-    NSInteger sdtype;
-
-
-NSArray *children;
-}
+@interface SideBarItem : NSObject
 
 @property (nonatomic, copy) NSString *title;
 @property (nonatomic, copy) NSString *identifier;
-@property (nonatomic, copy) NSString *untranslatedTitle;
 @property (nonatomic, retain) NSImage *icon;
 @property NSInteger badgeValue;
 @property NSInteger sdtype;
