@@ -1,10 +1,11 @@
 /*****************************************************************************
  * prefs.h: MacOS X module for vlc
  *****************************************************************************
- * Copyright (C) 2002-2007 VLC authors and VideoLAN
- * $Id: 3d1f644a62d6ffd2c04278833dcaed376d78c6e8 $
+ * Copyright (C) 2002-2015 VLC authors and VideoLAN
+ * $Id: 7d7e68f911e07a123cbd7873e89cbd77b8f1e8e5 $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
+ *          Felix Paul Kühne <fkuehne at videolan dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,42 +22,28 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+#import <Cocoa/Cocoa.h>
+
 @class VLCTreeMainItem;
 
 /*****************************************************************************
  * VLCPrefs interface
  *****************************************************************************/
-@interface VLCPrefs : NSObject
-{
-    intf_thread_t *p_intf;
-    VLCTreeMainItem * _rootTreeItem;
-    NSView *o_empty_view;
-    NSMutableDictionary *o_save_prefs;
+@interface VLCPrefs : NSWindowController<NSSplitViewDelegate>
 
-    IBOutlet id o_prefs_window;
-    IBOutlet id o_title;
-    IBOutlet id o_tree;
-    IBOutlet id o_prefs_view;
-    IBOutlet id o_save_btn;
-    IBOutlet id o_cancel_btn;
-    IBOutlet id o_reset_btn;
-    IBOutlet id o_showBasic_btn;
-}
+@property (readwrite, weak) IBOutlet NSTextField *titleLabel;
+@property (readwrite, weak) IBOutlet NSOutlineView *tree;
+@property (readwrite, weak) IBOutlet NSScrollView *prefsView;
+@property (readwrite, weak) IBOutlet NSButton *saveButton;
+@property (readwrite, weak) IBOutlet NSButton *cancelButton;
+@property (readwrite, weak) IBOutlet NSButton *resetButton;
+@property (readwrite, weak) IBOutlet NSButton *showBasicButton;
 
-+ (VLCPrefs *)sharedInstance;
-
-- (void)initStrings;
 - (void)setTitle: (NSString *) o_title_name;
 - (void)showPrefsWithLevel:(NSInteger)i_window_level;
 - (IBAction)savePrefs: (id)sender;
 - (IBAction)closePrefs: (id)sender;
-- (IBAction)buttonAction: (id)sender;
-
-@end
-
-@interface VLCFlippedView : NSView
-{
-
-}
+- (IBAction)showSimplePrefs: (id)sender;
+- (IBAction)resetPrefs: (id)sender;
 
 @end
