@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Copyright © 2011-2012 VideoLAN
- * $Id: 1314b520ea60eceabb83e0e90ddf07a98eccf9a2 $
+ * $Id: 7a63a9d7d9f13db7d68e426b448218ced3ae2b2d $
  *
  * Authors: Ludovic Fauvet <etix@l0cal.com>
  *
@@ -73,7 +73,7 @@ void TimeTooltip::adjustPosition()
 #if defined( Q_OS_WIN )
         mTarget.y() - 2 * size.height() - TIP_HEIGHT / 2 );
 #else
-        mTarget.y() - size.height() + TIP_HEIGHT / 2 );
+        mTarget.y() - size.height() - TIP_HEIGHT / 2 );
 #endif
 
     // Keep the tooltip on the same screen if possible
