@@ -2,7 +2,7 @@
  * playtree.cpp
  *****************************************************************************
  * Copyright (C) 2005 the VideoLAN team
- * $Id: 8abd634cf359e0253625b414f46f2d2be8b0df90 $
+ * $Id: c0d2fda468e0548ba22d234834edf872d4978884 $
  *
  * Authors: Antoine Cellerier <dionoea@videolan.org>
  *          Clément Stenac <zorglub@videolan.org>
@@ -36,7 +36,7 @@
 #include "../utils/ustring.hpp"
 
 Playtree::Playtree( intf_thread_t *pIntf )
-    : VarTree( pIntf ), m_pPlaylist( pIntf->p_sys->p_playlist )
+    : VarTree( pIntf ), m_pPlaylist( pl_Get(pIntf) )
 {
     getPositionVar().addObserver( this );
     buildTree();
@@ -59,15 +59,7 @@ void Playtree::delSelected()
                 playlist_ItemGetById( m_pPlaylist, it->getId() );
             if( pItem )
             {
-                if( pItem->i_children == -1 )
-                {
-                    playlist_DeleteFromInput( m_pPlaylist, pItem->p_input,
-                                              pl_Locked );
-                }
-                else
-                {
-                    playlist_NodeDelete( m_pPlaylist, pItem, true, false );
-                }
+                playlist_NodeDelete( m_pPlaylist, pItem );
             }
             playlist_Unlock( m_pPlaylist );
 
@@ -88,8 +80,7 @@ void Playtree::action( VarTree *pElem )
         playlist_ItemGetById( m_pPlaylist, pElem->getId() );
     if( pItem )
     {
-        playlist_Control( m_pPlaylist, PLAYLIST_VIEWPLAY,
-                          pl_Locked, pItem->p_parent, pItem );
+        playlist_ViewPlay( m_pPlaylist, pItem->p_parent, pItem );
     }
 
     playlist_Unlock( m_pPlaylist );
@@ -205,40 +196,43 @@ void Playtree::onDelete( int i_id )
     }
 }
 
-void Playtree::onAppend( playlist_add_t *p_add )
+void Playtree::onAppend( int i_id )
 {
-    Iterator it_node = findById( p_add->i_node );
-    if( it_node != m_children.end() )
+    playlist_item_t *pItem;
+
+    playlist_Lock( m_pPlaylist );
+    pItem = playlist_ItemGetById( m_pPlaylist, i_id );
+    if( !pItem || !pItem->p_parent )
     {
-        playlist_Lock( m_pPlaylist );
-        playlist_item_t *pItem =
-            playlist_ItemGetById( m_pPlaylist, p_add->i_item );
-        if( !pItem )
-        {
-            playlist_Unlock( m_pPlaylist );
-            return;
-        }
-
-        int pos;
-        for( pos = 0; pos < pItem->p_parent->i_children; pos++ )
-            if( pItem->p_parent->pp_children[pos] == pItem ) break;
-
-        UString *pName = getTitle( pItem->p_input );
-        playlist_item_t* current = playlist_CurrentPlayingItem( m_pPlaylist );
-
-        Iterator it = it_node->add(
-            p_add->i_item, UStringPtr( pName ), false, pItem == current,
-            false, pItem->i_flags & PLAYLIST_RO_FLAG, pos );
-
-        m_allItems[pItem->i_id] = &*it;
-
         playlist_Unlock( m_pPlaylist );
-
-        tree_update descr(
-            tree_update::ItemInserted,
-            IteratorVisible( it, this ) );
-        notify( &descr );
+        return;
     }
+
+    Iterator it_node = findById( pItem->p_parent->i_id );
+    if( it_node == m_children.end() )
+    {
+        playlist_Unlock( m_pPlaylist );
+        return;
+    }
+
+    int pos;
+    for( pos = 0; pos < pItem->p_parent->i_children; pos++ )
+        if( pItem->p_parent->pp_children[pos] == pItem ) break;
+
+    UString *pName = getTitle( pItem->p_input );
+    playlist_item_t* current = playlist_CurrentPlayingItem( m_pPlaylist );
+
+    Iterator it = it_node->add(
+        i_id, UStringPtr( pName ), false, pItem == current,
+        false, pItem->i_flags & PLAYLIST_RO_FLAG, pos );
+
+    m_allItems[i_id] = &*it;
+
+    playlist_Unlock( m_pPlaylist );
+
+    tree_update descr( tree_update::ItemInserted,
+                       IteratorVisible( it, this ) );
+    notify( &descr );
 }
 
 void Playtree::buildNode( playlist_item_t *pNode, VarTree &rTree )
@@ -261,9 +255,9 @@ void Playtree::buildTree()
     clear();
     playlist_Lock( m_pPlaylist );
 
-    for( int i = 0; i < m_pPlaylist->p_root->i_children; i++ )
+    for( int i = 0; i < m_pPlaylist->root.i_children; i++ )
     {
-        buildNode( m_pPlaylist->p_root->pp_children[i], *this );
+        buildNode( m_pPlaylist->root.pp_children[i], *this );
     }
 
     playlist_Unlock( m_pPlaylist );
@@ -277,7 +271,7 @@ void Playtree::onUpdateSlider()
 
 void Playtree::insertItems( VarTree& elem, const std::list<std::string>& files, bool start )
 {
-    bool first = true;
+    bool first = start;
     VarTree* p_elem = &elem;
     playlist_item_t* p_node = NULL;
     int i_pos = -1;
@@ -288,7 +282,7 @@ void Playtree::insertItems( VarTree& elem, const std::list<std::string>& files,
     {
         for( Iterator it = m_children.begin(); it != m_children.end(); ++it )
         {
-            if( it->getId() == m_pPlaylist->p_local_category->i_id )
+            if( it->getId() == m_pPlaylist->p_playing->i_id )
             {
                 p_elem = &*it;
                 break;
@@ -296,15 +290,15 @@ void Playtree::insertItems( VarTree& elem, const std::list<std::string>& files,
         }
     }
 
-    if( p_elem->getId() == m_pPlaylist->p_local_category->i_id )
+    if( p_elem->getId() == m_pPlaylist->p_playing->i_id )
     {
-        p_node = m_pPlaylist->p_local_category;
+        p_node = m_pPlaylist->p_playing;
         i_pos = 0;
         p_elem->setExpanded( true );
     }
-    else if( p_elem->getId() == m_pPlaylist->p_ml_category->i_id )
+    else if( p_elem->getId() == m_pPlaylist->p_media_library->i_id )
     {
-        p_node = m_pPlaylist->p_ml_category;
+        p_node = m_pPlaylist->p_media_library;
         i_pos = 0;
         p_elem->setExpanded( true );
     }
@@ -325,9 +319,10 @@ void Playtree::insertItems( VarTree& elem, const std::list<std::string>& files,
         goto fin;
 
     for( std::list<std::string>::const_iterator it = files.begin();
-         it != files.end(); ++it, i_pos++, first = false )
+         it != files.end(); ++it, i_pos++ )
     {
         input_item_t *pItem;
+        playlist_item_t *pPlItem;
 
         if( strstr( it->c_str(), "://" ) )
             pItem = input_item_New( it->c_str(), NULL );
@@ -343,12 +338,13 @@ void Playtree::insertItems( VarTree& elem, const std::list<std::string>& files,
         if( pItem == NULL)
             continue;
 
-        int i_mode = PLAYLIST_APPEND;
-        if( first && start )
-            i_mode |= PLAYLIST_GO;
+        pPlItem = playlist_NodeAddInput( m_pPlaylist, pItem, p_node, i_pos );
 
-        playlist_NodeAddInput( m_pPlaylist, pItem, p_node,
-                               i_mode, i_pos, pl_Locked );
+        if( likely(pPlItem != NULL) && first )
+        {
+            first = false;
+            playlist_ViewPlay( m_pPlaylist, NULL, pPlItem );
+        }
     }
 
 fin:
