@@ -2,7 +2,7 @@
  * x11_dragdrop.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: 36ef6b792ab0fdbcb36a428954796357834b7aab $
+ * $Id: 9244a2532e9c167023384abeba9489e187e83f1f $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -48,10 +48,12 @@ X11DragDrop::X11DragDrop( intf_thread_t *pIntf, X11Display &rDisplay,
 void X11DragDrop::dndEnter( ldata_t data )
 {
     Window src = data[0];
+    int version = data[1]>>24;
     m_xPos = m_yPos = -1;
+    (void)version;
 
     // Retrieve available data types
-    list<string> dataTypes;
+    std::list<std::string> dataTypes;
     if( data[1] & 1 )   // More than 3 data types ?
     {
         Atom type;
@@ -64,7 +66,7 @@ void X11DragDrop::dndEnter( ldata_t data )
                             (unsigned char**)&dataList );
         for( unsigned long i=0; i<nitems; i++ )
         {
-            string dataType = XGetAtomName( XDISPLAY, dataList[i] );
+            std::string dataType = XGetAtomName( XDISPLAY, dataList[i] );
             dataTypes.push_back( dataType );
         }
         XFree( (void*)dataList );
@@ -75,22 +77,29 @@ void X11DragDrop::dndEnter( ldata_t data )
         {
             if( data[i] != None )
             {
-                string dataType = XGetAtomName( XDISPLAY, data[i] );
+                std::string dataType = XGetAtomName( XDISPLAY, data[i] );
                 dataTypes.push_back( dataType );
             }
         }
     }
 
+    // list all data types available
+    std::list<std::string>::iterator it;
+    for( it = dataTypes.begin(); it != dataTypes.end(); ++it )
+        msg_Dbg( getIntf(), "D&D data type: %s", (*it).c_str() );
+
     // Find the right target
     m_target = None;
-    list<string>::iterator it;
     for( it = dataTypes.begin(); it != dataTypes.end(); ++it )
     {
         if( *it == "text/uri-list" ||
             *it == "text/plain" ||
-            *it == "STRING" )
+            *it == "text/plain;charset=utf-8" ||
+            *it == "STRING" ||
+            *it == "UTF8_STRING" )
         {
             m_target = XInternAtom( XDISPLAY, (*it).c_str(), 0 );
+            msg_Dbg( getIntf(), "D&D data type chosen: %s", (*it).c_str() );
             break;
         }
     }
@@ -107,22 +116,11 @@ void X11DragDrop::dndPosition( ldata_t data )
     m_xPos = data[2] >> 16;
     m_yPos = data[2] & 0xffff;
     Time time = data[3];
-
-    Atom selectionAtom = XInternAtom( XDISPLAY, "XdndSelection", 0 );
-    //Atom targetAtom = XInternAtom( XDISPLAY, "text/plain", 0 );
-    Atom targetAtom = XInternAtom( XDISPLAY, "text/uri-list", 0 );
-    Atom propAtom = XInternAtom( XDISPLAY, "VLC_SELECTION", 0 );
+    Atom action = data[4];
+    (void)time; (void)action;
 
     Atom actionAtom = XInternAtom( XDISPLAY, "XdndActionCopy", 0 );
-    Atom typeAtom = XInternAtom( XDISPLAY, "XdndFinished", 0 );
-
-    // Convert the selection into the given target
-    // NEEDED or it doesn't work!
-    XConvertSelection( XDISPLAY, selectionAtom, targetAtom, propAtom, src,
-                       time );
-
-    actionAtom = XInternAtom( XDISPLAY, "XdndActionCopy", 0 );
-    typeAtom = XInternAtom( XDISPLAY, "XdndStatus", 0 );
+    Atom typeAtom = XInternAtom( XDISPLAY, "XdndStatus", 0 );
 
     XEvent event;
     event.type = ClientMessage;
@@ -149,6 +147,8 @@ void X11DragDrop::dndPosition( ldata_t data )
 void X11DragDrop::dndLeave( ldata_t data )
 {
     (void)data;
+    m_target = None;
+
     // transmit DragLeave event
     EvtDragLeave evt( getIntf() );
     m_pWin->processEvent( evt );
@@ -157,32 +157,39 @@ void X11DragDrop::dndLeave( ldata_t data )
 
 void X11DragDrop::dndDrop( ldata_t data )
 {
-    list<string> files;
+    std::list<std::string> files;
 
     Window src = data[0];
     Time time = data[2];
 
     Atom selectionAtom = XInternAtom( XDISPLAY, "XdndSelection", 0 );
-    Atom targetAtom = XInternAtom( XDISPLAY, "text/uri-list", 0 );
     Atom propAtom = XInternAtom( XDISPLAY, "VLC_SELECTION", 0 );
-
-    Atom actionAtom = XInternAtom( XDISPLAY, "XdndActionCopy", 0 );
-    Atom typeAtom = XInternAtom( XDISPLAY, "XdndFinished", 0 );
-
     // Convert the selection into the given target
-    XConvertSelection( XDISPLAY, selectionAtom, targetAtom, propAtom, src,
+    XConvertSelection( XDISPLAY, selectionAtom, m_target, propAtom, src,
                        time );
+    // Needed to ensure XGetWindowProperty returns something
+    XSync( XDISPLAY, False );
 
     // Read the selection
     Atom type;
     int format;
-    unsigned long nitems, nbytes;
+    unsigned long nitems, nbytes_after_return;
     char *buffer;
-    XGetWindowProperty( XDISPLAY, src, propAtom, 0, 1024, False,
-                        AnyPropertyType, &type, &format, &nitems, &nbytes,
-                        (unsigned char**)&buffer );
+    long length_max = 1024;
+    XGetWindowProperty( XDISPLAY, src, propAtom, 0, length_max, False,
+                        AnyPropertyType, &type, &format, &nitems,
+                        &nbytes_after_return, (unsigned char**)&buffer );
+    if( buffer && nbytes_after_return > 0 )
+    {
+        XFree( buffer );
+        length_max += nbytes_after_return;
+        XGetWindowProperty( XDISPLAY, src, propAtom, 0, length_max, False,
+                        AnyPropertyType, &type, &format, &nitems,
+                        &nbytes_after_return, (unsigned char**)&buffer );
+    }
     if( buffer != NULL )
     {
+        msg_Dbg( getIntf(), "buffer received: %s", buffer );
         char* psz_dup = strdup( buffer );
         char* psz_new = psz_dup;
         while( psz_new && *psz_new )
@@ -198,7 +205,7 @@ void X11DragDrop::dndDrop( ldata_t data )
                 skip = strlen( sep[i] );
                 break;
             }
-            if( *psz_new )
+            if( *psz_new && strstr( psz_new, "://" ) )
             {
                 files.push_back( psz_new );
             }
@@ -209,6 +216,9 @@ void X11DragDrop::dndDrop( ldata_t data )
         XFree( buffer );
     }
 
+    Atom actionAtom = XInternAtom( XDISPLAY, "XdndActionCopy", 0 );
+    Atom typeAtom = XInternAtom( XDISPLAY, "XdndFinished", 0 );
+
     // Tell the source we accepted the drop
     XEvent event;
     event.type = ClientMessage;
