@@ -2,7 +2,7 @@
  * algo_yadif.c : Wrapper for FFmpeg's Yadif algorithm
  *****************************************************************************
  * Copyright (C) 2000-2011 VLC authors and VideoLAN
- * $Id: d94e45dbb5276ee99d193747d74f0144d61ff791 $
+ * $Id: a89d240b831419267cc8f031f2b1909eed44db89 $
  *
  * Author: Laurent Aimar <fenrir@videolan.org>
  *         Juha Jeronen  <juha.jeronen@jyu.fi> (soft field repeat hack)
@@ -113,20 +113,23 @@ int RenderYadif( filter_t *p_filter, picture_t *p_dst, picture_t *p_src,
         void (*filter)(uint8_t *dst, uint8_t *prev, uint8_t *cur, uint8_t *next,
                        int w, int prefs, int mrefs, int parity, int mode);
 
-#if defined(HAVE_YADIF_SSSE3)
+/* android clang build for x86 fails as not enough registers are available */
+#if !defined(__ANDROID__)
+# if defined(HAVE_YADIF_SSSE3)
         if( vlc_CPU_SSSE3() )
             filter = yadif_filter_line_ssse3;
         else
-#endif
-#if defined(HAVE_YADIF_SSE2)
+# endif
+# if defined(HAVE_YADIF_SSE2)
         if( vlc_CPU_SSE2() )
             filter = yadif_filter_line_sse2;
         else
-#endif
-#if defined(HAVE_YADIF_MMX)
+# endif
+# if defined(HAVE_YADIF_MMX)
         if( vlc_CPU_MMX() )
             filter = yadif_filter_line_mmx;
         else
+# endif
 #endif
             filter = yadif_filter_line_c;
 
