@@ -2,7 +2,7 @@
  * common.c: Windows video output common code
  *****************************************************************************
  * Copyright (C) 2001-2009 VLC authors and VideoLAN
- * $Id: 886d57986fbad84397b960f6b37d663de511d465 $
+ * $Id: ca7c5401e6563da40ac8501869b19413aa98a69c $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Martell Malone <martellmalone@gmail.com>
@@ -41,16 +41,36 @@
 #define COBJMACROS
 #include <shobjidl.h>
 
+#define vout_display_sys_win32_t vout_display_sys_t
+
 #include "common.h"
+#include "../video_chroma/copy.h"
+
+#if !defined(NDEBUG) && defined(HAVE_DXGIDEBUG_H)
+# include <initguid.h>
+# include <dxgidebug.h>
+#endif
 
 static void CommonChangeThumbnailClip(vout_display_t *, bool show);
 #if !VLC_WINSTORE_APP
 static int  CommonControlSetFullscreen(vout_display_t *, bool is_fullscreen);
 
-static void DisableScreensaver(vout_display_t *);
-static void RestoreScreensaver(vout_display_t *);
+static bool GetRect(const vout_display_sys_t *sys, RECT *out)
+{
+    return GetClientRect(sys->hwnd, out);
+}
 #endif
 
+static unsigned int GetPictureWidth(const vout_display_t *vd)
+{
+    return vd->fmt.i_width;
+}
+
+static unsigned int GetPictureHeight(const vout_display_t *vd)
+{
+    return vd->fmt.i_height;
+}
+
 /* */
 int CommonInit(vout_display_t *vd)
 {
@@ -64,7 +84,10 @@ int CommonInit(vout_display_t *vd)
     sys->is_first_display = true;
     sys->is_on_top        = false;
 
+    sys->pf_GetPictureWidth  = GetPictureWidth;
+    sys->pf_GetPictureHeight = GetPictureHeight;
 #if !VLC_WINSTORE_APP
+    sys->pf_GetRect = GetRect;
     SetRectEmpty(&sys->rect_display);
     SetRectEmpty(&sys->rect_parent);
 
@@ -77,6 +100,7 @@ int CommonInit(vout_display_t *vd)
         return VLC_EGENERIC;
 #endif
 
+#if !VLC_WINSTORE_APP
     event_cfg_t cfg;
     memset(&cfg, 0, sizeof(cfg));
 #ifdef MODULE_NAME_IS_direct3d9
@@ -90,7 +114,6 @@ int CommonInit(vout_display_t *vd)
     cfg.width  = vd->cfg->display.width;
     cfg.height = vd->cfg->display.height;
 
-#if !VLC_WINSTORE_APP
     event_hwnd_t hwnd;
     if (EventThreadStart(sys->event, &hwnd, &cfg))
         return VLC_EGENERIC;
@@ -103,10 +126,12 @@ int CommonInit(vout_display_t *vd)
 
     if (vd->cfg->is_fullscreen) {
         if (CommonControlSetFullscreen(vd, true))
-            vout_display_SendEventFullscreen(vd, false);
+            vout_display_SendEventFullscreen(vd, false, false);
     }
 
-    DisableScreensaver (vd);
+#endif
+#if !defined(NDEBUG) && defined(HAVE_DXGIDEBUG_H)
+    sys->dxgidebug_dll = LoadLibrary(TEXT("DXGIDEBUG.DLL"));
 #endif
 
     return VLC_SUCCESS;
@@ -128,10 +153,10 @@ picture_pool_t *CommonPool(vout_display_t *vd, unsigned count)
 *****************************************************************************/
 void UpdateRects(vout_display_t *vd,
     const vout_display_cfg_t *cfg,
-    const video_format_t *source,
     bool is_forced)
 {
     vout_display_sys_t *sys = vd->sys;
+    const video_format_t *source = &vd->source;
 #define rect_src sys->rect_src
 #define rect_src_clipped sys->rect_src_clipped
 #define rect_dest sys->rect_dest
@@ -143,32 +168,10 @@ void UpdateRects(vout_display_t *vd,
     /* */
     if (!cfg)
         cfg = vd->cfg;
-    if (!source)
-        source = &vd->source;
 
     /* Retrieve the window size */
-#if VLC_WINSTORE_APP && MODULE_NAME_IS_direct3d11
-    rect.left   = 0;
-    rect.top    = 0;
-    uint32_t i_width;
-    uint32_t i_height;
-    UINT dataSize = sizeof(i_width);
-    HRESULT hr = IDXGISwapChain_GetPrivateData(sys->dxgiswapChain, &GUID_SWAPCHAIN_WIDTH, &dataSize, &i_width);
-    if (FAILED(hr)) {
-        msg_Err(vd, "Can't get swapchain width, size %d. (hr=0x%lX)", hr, dataSize);
+    if (!sys->pf_GetRect(sys, &rect))
         return;
-    }
-    dataSize = sizeof(i_height);
-    hr = IDXGISwapChain_GetPrivateData(sys->dxgiswapChain, &GUID_SWAPCHAIN_HEIGHT, &dataSize, &i_height);
-    if (FAILED(hr)) {
-        msg_Err(vd, "Can't get swapchain height, size %d. (hr=0x%lX)", hr, dataSize);
-        return;
-    }
-    rect.right  = i_width;
-    rect.bottom = i_height;
-#else
-    GetClientRect(sys->hwnd, &rect);
-#endif
 
     /* Retrieve the window position */
     point.x = point.y = 0;
@@ -199,6 +202,14 @@ void UpdateRects(vout_display_t *vd,
     place_cfg.display.width = rect.right;
     place_cfg.display.height = rect.bottom;
 
+#if (defined(MODULE_NAME_IS_glwin32))
+    /* Reverse vertical alignment as the GL tex are Y inverted */
+    if (place_cfg.align.vertical == VOUT_DISPLAY_ALIGN_TOP)
+        place_cfg.align.vertical = VOUT_DISPLAY_ALIGN_BOTTOM;
+    else if (place_cfg.align.vertical == VOUT_DISPLAY_ALIGN_BOTTOM)
+        place_cfg.align.vertical = VOUT_DISPLAY_ALIGN_TOP;
+#endif
+
     vout_display_place_t place;
     vout_display_PlacePicture(&place, source, &place_cfg, false);
 
@@ -212,7 +223,7 @@ void UpdateRects(vout_display_t *vd,
 #endif
 
     /* Destination image position and dimensions */
-#if (defined(MODULE_NAME_IS_direct3d9) || defined(MODULE_NAME_IS_direct3d11) || defined(MODULE_NAME_IS_direct2d)) && !VLC_WINSTORE_APP
+#if (defined(MODULE_NAME_IS_direct3d9) || defined(MODULE_NAME_IS_direct3d11)) && !VLC_WINSTORE_APP
     rect_dest.left = 0;
     rect_dest.right = place.width;
     rect_dest.top = 0;
@@ -268,8 +279,8 @@ void UpdateRects(vout_display_t *vd,
     /* src image dimensions */
     rect_src.left = 0;
     rect_src.top = 0;
-    rect_src.right = vd->fmt.i_width;
-    rect_src.bottom = vd->fmt.i_height;
+    rect_src.right = sys->pf_GetPictureWidth(vd);
+    rect_src.bottom = sys->pf_GetPictureHeight(vd);
 
     /* Clip the source image */
     rect_src_clipped.left = source->i_x_offset +
@@ -291,16 +302,10 @@ void UpdateRects(vout_display_t *vd,
     /* Apply overlay hardware constraints */
     if (sys->use_overlay)
         AlignRect(&rect_src_clipped, sys->i_align_src_boundary, sys->i_align_src_size);
-#elif defined(MODULE_NAME_IS_direct3d9) || defined(MODULE_NAME_IS_direct3d11) || defined(MODULE_NAME_IS_direct2d)
-    /* Needed at least with YUV content */
-    rect_src_clipped.left &= ~1;
-    rect_src_clipped.right &= ~1;
-    rect_src_clipped.top &= ~1;
-    rect_src_clipped.bottom &= ~1;
 #endif
 
 #ifndef NDEBUG
-    msg_Dbg(vd, "DirectXUpdateRects souce"
+    msg_Dbg(vd, "DirectXUpdateRects source"
         " offset: %i,%i visible: %ix%i",
         source->i_x_offset, source->i_y_offset,
         source->i_visible_width, source->i_visible_height);
@@ -354,7 +359,20 @@ void CommonClean(vout_display_t *vd)
         EventThreadDestroy(sys->event);
     }
 
-    RestoreScreensaver(vd);
+#if !defined(NDEBUG) && defined(HAVE_DXGIDEBUG_H)
+    HRESULT (WINAPI  * pf_DXGIGetDebugInterface)(const GUID *riid, void **ppDebug);
+    if (sys->dxgidebug_dll) {
+        pf_DXGIGetDebugInterface = (void *)GetProcAddress(sys->dxgidebug_dll, "DXGIGetDebugInterface");
+        if (pf_DXGIGetDebugInterface) {
+            IDXGIDebug *pDXGIDebug = NULL;
+            HRESULT hr = pf_DXGIGetDebugInterface(&IID_IDXGIDebug, (void**)&pDXGIDebug);
+            if (SUCCEEDED(hr) && pDXGIDebug) {
+                hr = IDXGIDebug_ReportLiveObjects(pDXGIDebug, DXGI_DEBUG_ALL, DXGI_DEBUG_RLO_ALL);
+            }
+        }
+        FreeLibrary(sys->dxgidebug_dll);
+    }
+#endif
 }
 
 void CommonManage(vout_display_t *vd)
@@ -394,13 +412,13 @@ void CommonManage(vout_display_t *vd)
                          rect_parent.bottom - rect_parent.top,
                          SWP_NOZORDER);
 
-            UpdateRects(vd, NULL, NULL, true);
+            UpdateRects(vd, NULL, true);
         }
     }
 
     /* HasMoved means here resize or move */
     if (EventThreadGetAndResetHasMoved(sys->event))
-        UpdateRects(vd, NULL, NULL, false);
+        UpdateRects(vd, NULL, false);
 }
 
 /**
@@ -448,51 +466,7 @@ int CommonUpdatePicture(picture_t *picture, picture_t **fallback,
         }
         return VLC_SUCCESS;
     }
-    /* fill in buffer info in first plane */
-    picture->p->p_pixels = data;
-    picture->p->i_pitch  = pitch;
-    picture->p->i_lines  = picture->format.i_height;
-
-    /*  Fill chroma planes for biplanar YUV */
-    if (picture->format.i_chroma == VLC_CODEC_NV12 ||
-        picture->format.i_chroma == VLC_CODEC_NV21) {
-
-        for (int n = 1; n < picture->i_planes; n++) {
-            const plane_t *o = &picture->p[n-1];
-            plane_t *p = &picture->p[n];
-
-            p->p_pixels = o->p_pixels + o->i_lines * o->i_pitch;
-            p->i_pitch  = pitch;
-            p->i_lines  = picture->format.i_height;
-        }
-        /* The dx/d3d buffer is always allocated as NV12 */
-        if (vlc_fourcc_AreUVPlanesSwapped(picture->format.i_chroma, VLC_CODEC_NV12)) {
-            /* TODO : Swap NV21 UV planes to match NV12 */
-            return VLC_EGENERIC;
-        }
-    }
-
-    /*  Fill chroma planes for planar YUV */
-    if (picture->format.i_chroma == VLC_CODEC_I420 ||
-        picture->format.i_chroma == VLC_CODEC_J420 ||
-        picture->format.i_chroma == VLC_CODEC_YV12) {
-
-        for (int n = 1; n < picture->i_planes; n++) {
-            const plane_t *o = &picture->p[n-1];
-            plane_t *p = &picture->p[n];
-
-            p->p_pixels = o->p_pixels + o->i_lines * o->i_pitch;
-            p->i_pitch  = pitch / 2;
-            p->i_lines  = picture->format.i_height / 2;
-        }
-        /* The dx/d3d buffer is always allocated as YV12 */
-        if (vlc_fourcc_AreUVPlanesSwapped(picture->format.i_chroma, VLC_CODEC_YV12)) {
-            uint8_t *p_tmp = picture->p[1].p_pixels;
-            picture->p[1].p_pixels = picture->p[2].p_pixels;
-            picture->p[2].p_pixels = p_tmp;
-        }
-    }
-    return VLC_SUCCESS;
+    return picture_UpdatePlanes(picture, data, pitch);
 }
 
 void AlignRect(RECT *r, int align_boundary, int align_size)
@@ -527,20 +501,25 @@ static void CommonChangeThumbnailClip(vout_display_t *vd, bool show)
         taskbl->lpVtbl->HrInit(taskbl);
 
         HWND hroot = GetAncestor(sys->hwnd,GA_ROOT);
-        RECT relative;
+        RECT video;
         if (show) {
-            RECT video, parent;
-            GetWindowRect(sys->hvideownd, &video);
-            GetWindowRect(hroot, &parent);
-            relative.left   = video.left   - parent.left - 8;
-            relative.top    = video.top    - parent.top - 10;
-
-            relative.right  = video.right  - video.left + relative.left;
-            relative.bottom = video.bottom - video.top  + relative.top - 25;
+            GetWindowRect(sys->hparent, &video);
+            POINT client = {video.left, video.top};
+            if (ScreenToClient(hroot, &client))
+            {
+                unsigned int width = video.right - video.left;
+                unsigned int height = video.bottom - video.top;
+                video.left = client.x;
+                video.top = client.y;
+                video.right = video.left + width;
+                video.bottom = video.top + height;
+            }
         }
-        if (S_OK != taskbl->lpVtbl->SetThumbnailClip(taskbl, hroot,
-                                                 show ? &relative : NULL))
-            msg_Err(vd, "SetThumbNailClip failed");
+        HRESULT hr;
+        hr = taskbl->lpVtbl->SetThumbnailClip(taskbl, hroot,
+                                                 show ? &video : NULL);
+        if ( hr != S_OK )
+            msg_Err(vd, "SetThumbNailClip failed: 0x%0lx", hr);
 
         taskbl->lpVtbl->Release(taskbl);
     }
@@ -634,11 +613,37 @@ static int CommonControlSetFullscreen(vout_display_t *vd, bool is_fullscreen)
     return VLC_SUCCESS;
 }
 
+#else
+
+void CommonManage(vout_display_t *vd) {
+    UpdateRects(vd, NULL, false);
+}
+void CommonClean(vout_display_t *vd) {}
+void CommonDisplay(vout_display_t *vd) {}
+void CommonChangeThumbnailClip(vout_display_t *vd, bool show) {}
+#endif
+
 int CommonControl(vout_display_t *vd, int query, va_list args)
 {
     vout_display_sys_t *sys = vd->sys;
 
     switch (query) {
+    case VOUT_DISPLAY_CHANGE_DISPLAY_FILLED: /* const vout_display_cfg_t *p_cfg */
+    case VOUT_DISPLAY_CHANGE_ZOOM:           /* const vout_display_cfg_t *p_cfg */
+    case VOUT_DISPLAY_CHANGE_SOURCE_ASPECT:
+    case VOUT_DISPLAY_CHANGE_SOURCE_CROP: {
+        const vout_display_cfg_t *cfg;
+
+        if (query == VOUT_DISPLAY_CHANGE_SOURCE_CROP ||
+            query == VOUT_DISPLAY_CHANGE_SOURCE_ASPECT) {
+            cfg    = vd->cfg;
+        } else {
+            cfg    = va_arg(args, const vout_display_cfg_t *);
+        }
+        UpdateRects(vd, cfg, true);
+        return VLC_SUCCESS;
+    }
+#if !VLC_WINSTORE_APP
     case VOUT_DISPLAY_CHANGE_DISPLAY_SIZE:   /* const vout_display_cfg_t *p_cfg */
     {   /* Update dimensions */
         const vout_display_cfg_t *cfg = va_arg(args, const vout_display_cfg_t *);
@@ -649,28 +654,13 @@ int CommonControl(vout_display_t *vd, int query, va_list args)
             .bottom = cfg->display.height,
         };
 
-        AdjustWindowRect(&rect_window, EventThreadGetWindowStyle(sys->event), 0);
-        SetWindowPos(sys->hwnd, 0, 0, 0,
-                     rect_window.right - rect_window.left,
-                     rect_window.bottom - rect_window.top, SWP_NOMOVE);
-        UpdateRects(vd, cfg, NULL, false);
-        return VLC_SUCCESS;
-    }
-    case VOUT_DISPLAY_CHANGE_DISPLAY_FILLED: /* const vout_display_cfg_t *p_cfg */
-    case VOUT_DISPLAY_CHANGE_ZOOM:           /* const vout_display_cfg_t *p_cfg */
-    case VOUT_DISPLAY_CHANGE_SOURCE_ASPECT:  /* const video_format_t *p_source */
-    case VOUT_DISPLAY_CHANGE_SOURCE_CROP: {  /* const video_format_t *p_source */
-        const vout_display_cfg_t *cfg;
-
-        if (query == VOUT_DISPLAY_CHANGE_SOURCE_CROP ||
-            query == VOUT_DISPLAY_CHANGE_SOURCE_ASPECT) {
-            const video_format_t *source = va_arg(args, const video_format_t *);
-            cfg    = vd->cfg;
-            UpdateRects(vd, cfg, source, true);
-        } else {
-            cfg    = va_arg(args, const vout_display_cfg_t *);
-            UpdateRects(vd, cfg, NULL, false);
+        if (!cfg->is_fullscreen) {
+            AdjustWindowRect(&rect_window, EventThreadGetWindowStyle(sys->event), 0);
+            SetWindowPos(sys->hwnd, 0, 0, 0,
+                         rect_window.right - rect_window.left,
+                         rect_window.bottom - rect_window.top, SWP_NOMOVE);
         }
+        UpdateRects(vd, cfg, false);
         return VLC_SUCCESS;
     }
     case VOUT_DISPLAY_CHANGE_WINDOW_STATE: {       /* unsigned state */
@@ -696,62 +686,14 @@ int CommonControl(vout_display_t *vd, int query, va_list args)
         bool fs = va_arg(args, int);
         if (CommonControlSetFullscreen(vd, fs))
             return VLC_EGENERIC;
-        UpdateRects(vd, NULL, NULL, false);
+        UpdateRects(vd, NULL, false);
         return VLC_SUCCESS;
     }
 
-    case VOUT_DISPLAY_HIDE_MOUSE:
-        EventThreadMouseHide(sys->event);
-        return VLC_SUCCESS;
     case VOUT_DISPLAY_RESET_PICTURES:
         vlc_assert_unreachable();
-    default:
-        return VLC_EGENERIC;
-    }
-}
-
-static void DisableScreensaver(vout_display_t *vd)
-{
-    vout_display_sys_t *sys = vd->sys;
-
-    /* disable screensaver by temporarily changing system settings */
-    sys->i_spi_screensaveactive = 0;
-    if (var_GetBool(vd, "disable-screensaver")) {
-        msg_Dbg(vd, "disabling screen saver");
-        SystemParametersInfo(SPI_GETSCREENSAVEACTIVE, 0,
-                             &sys->i_spi_screensaveactive, 0);
-
-        if (FALSE != sys->i_spi_screensaveactive) {
-            SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, 0, NULL, 0);
-        }
-    }
-}
-
-static void RestoreScreensaver(vout_display_t *vd)
-{
-    vout_display_sys_t *sys = vd->sys;
-
-    /* restore screensaver system settings */
-    if (0 != sys->i_spi_screensaveactive) {
-        SystemParametersInfo(SPI_SETSCREENSAVEACTIVE,
-                             sys->i_spi_screensaveactive, NULL, 0);
-    }
-}
-
-#else
-
-int CommonControl(vout_display_t *vd, int query, va_list args)
-{
-    switch (query) {
-    default:
-        return VLC_EGENERIC;
-    }
-}
-
-void CommonManage(vout_display_t *vd) {
-    UpdateRects(vd, NULL, NULL, false);
-}
-void CommonClean(vout_display_t *vd) {}
-void CommonDisplay(vout_display_t *vd) {}
-void CommonChangeThumbnailClip(vout_display_t *vd, bool show) {}
 #endif
+    default:
+        return VLC_EGENERIC;
+    }
+}
