@@ -2,7 +2,7 @@
  * var.c: object variables for input thread
  *****************************************************************************
  * Copyright (C) 2004-2007 VLC authors and VideoLAN
- * $Id: 65346038490a99379956d1be73166f8f03f7d877 $
+ * $Id: 631b571c19136a9aded81153190972cd92613619 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -57,8 +57,12 @@ static int SeekpointCallback( vlc_object_t *p_this, char const *psz_cmd,
                              vlc_value_t oldval, vlc_value_t newval, void * );
 static int NavigationCallback( vlc_object_t *p_this, char const *psz_cmd,
                              vlc_value_t oldval, vlc_value_t newval, void * );
-static int ESCallback      ( vlc_object_t *p_this, char const *psz_cmd,
+static int EsVideoCallback ( vlc_object_t *p_this, char const *psz_cmd,
                              vlc_value_t oldval, vlc_value_t newval, void * );
+static int EsAudioCallback ( vlc_object_t *p_this, char const *psz_cmd,
+                             vlc_value_t oldval, vlc_value_t newval, void * );
+static int EsSpuCallback ( vlc_object_t *p_this, char const *psz_cmd,
+                           vlc_value_t oldval, vlc_value_t newval, void * );
 static int EsDelayCallback ( vlc_object_t *p_this, char const *psz_cmd,
                              vlc_value_t oldval, vlc_value_t newval, void * );
 
@@ -91,7 +95,6 @@ static const vlc_input_callback_t p_input_callbacks[] =
     CALLBACK( "state", StateCallback ),
     CALLBACK( "rate", RateCallback ),
     CALLBACK( "position", PositionCallback ),
-    CALLBACK( "position-offset", PositionCallback ),
     CALLBACK( "time", TimeCallback ),
     CALLBACK( "time-offset", TimeOffsetCallback ),
     CALLBACK( "bookmark", BookmarkCallback ),
@@ -100,15 +103,15 @@ static const vlc_input_callback_t p_input_callbacks[] =
     CALLBACK( "chapter", SeekpointCallback ),
     CALLBACK( "audio-delay", EsDelayCallback ),
     CALLBACK( "spu-delay", EsDelayCallback ),
-    CALLBACK( "video-es", ESCallback ),
-    CALLBACK( "audio-es", ESCallback ),
-    CALLBACK( "spu-es", ESCallback ),
+    CALLBACK( "video-es", EsVideoCallback ),
+    CALLBACK( "audio-es", EsAudioCallback ),
+    CALLBACK( "spu-es", EsSpuCallback ),
     CALLBACK( "record", RecordCallback ),
     CALLBACK( "frame-next", FrameNextCallback ),
 
     CALLBACK( NULL, NULL )
 };
-static const vlc_input_callback_t p_input_navigation_callbacks[] =
+static const vlc_input_callback_t p_input_title_navigation_callbacks[] =
 {
     CALLBACK( "next-title", TitleCallback ),
     CALLBACK( "prev-title", TitleCallback ),
@@ -117,13 +120,6 @@ static const vlc_input_callback_t p_input_navigation_callbacks[] =
 
     CALLBACK( NULL, NULL )
 };
-static const vlc_input_callback_t p_input_title_callbacks[] =
-{
-    CALLBACK( "next-chapter", SeekpointCallback ),
-    CALLBACK( "prev-chapter", SeekpointCallback ),
-
-    CALLBACK( NULL, NULL )
-};
 #undef CALLBACK
 
 /*****************************************************************************
@@ -136,7 +132,7 @@ void input_ControlVarInit ( input_thread_t *p_input )
 
     /* State */
     var_Create( p_input, "state", VLC_VAR_INTEGER );
-    val.i_int = p_input->p->i_state;
+    val.i_int = input_priv(p_input)->i_state;
     var_Change( p_input, "state", VLC_VAR_SETVALUE, &val, NULL );
 
     /* Rate */
@@ -146,21 +142,18 @@ void input_ControlVarInit ( input_thread_t *p_input )
 
     /* Position */
     var_Create( p_input, "position",  VLC_VAR_FLOAT );
-    var_Create( p_input, "position-offset",  VLC_VAR_FLOAT );
 
     /* Time */
     var_Create( p_input, "time", VLC_VAR_INTEGER );
     var_Create( p_input, "time-offset", VLC_VAR_INTEGER );    /* relative */
 
     /* Bookmark */
-    var_Create( p_input, "bookmark", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE |
-                VLC_VAR_ISCOMMAND );
+    var_Create( p_input, "bookmark", VLC_VAR_INTEGER | VLC_VAR_ISCOMMAND );
     val.psz_string = _("Bookmark");
     var_Change( p_input, "bookmark", VLC_VAR_SETTEXT, &val, NULL );
 
     /* Program */
-    var_Create( p_input, "program", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE |
-                VLC_VAR_DOINHERIT );
+    var_Create( p_input, "program", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
     var_Get( p_input, "program", &val );
     if( val.i_int <= 0 )
         var_Change( p_input, "program", VLC_VAR_DELCHOICE, &val, NULL );
@@ -173,12 +166,12 @@ void input_ControlVarInit ( input_thread_t *p_input )
     var_Change( p_input, "programs", VLC_VAR_SETTEXT, &text, NULL );
 
     /* Title */
-    var_Create( p_input, "title", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
+    var_Create( p_input, "title", VLC_VAR_INTEGER );
     text.psz_string = _("Title");
     var_Change( p_input, "title", VLC_VAR_SETTEXT, &text, NULL );
 
     /* Chapter */
-    var_Create( p_input, "chapter", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
+    var_Create( p_input, "chapter", VLC_VAR_INTEGER );
     text.psz_string = _("Chapter");
     var_Change( p_input, "chapter", VLC_VAR_SETTEXT, &text, NULL );
 
@@ -188,18 +181,22 @@ void input_ControlVarInit ( input_thread_t *p_input )
                     1000 * var_GetInteger( p_input, "audio-desync" ) );
     var_Create( p_input, "spu-delay", VLC_VAR_INTEGER );
 
+    val.i_int = -1;
     /* Video ES */
-    var_Create( p_input, "video-es", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
+    var_Create( p_input, "video-es", VLC_VAR_INTEGER );
+    var_Change( p_input, "video-es", VLC_VAR_SETVALUE, &val, NULL );
     text.psz_string = _("Video Track");
     var_Change( p_input, "video-es", VLC_VAR_SETTEXT, &text, NULL );
 
     /* Audio ES */
-    var_Create( p_input, "audio-es", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
+    var_Create( p_input, "audio-es", VLC_VAR_INTEGER );
+    var_Change( p_input, "audio-es", VLC_VAR_SETVALUE, &val, NULL );
     text.psz_string = _("Audio Track");
     var_Change( p_input, "audio-es", VLC_VAR_SETTEXT, &text, NULL );
 
     /* Spu ES */
-    var_Create( p_input, "spu-es", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
+    var_Create( p_input, "spu-es", VLC_VAR_INTEGER );
+    var_Change( p_input, "spu-es", VLC_VAR_SETVALUE, &val, NULL );
     text.psz_string = _("Subtitle Track");
     var_Change( p_input, "spu-es", VLC_VAR_SETTEXT, &text, NULL );
 
@@ -220,7 +217,7 @@ void input_ControlVarInit ( input_thread_t *p_input )
     /* Add all callbacks
      * XXX we put callback only in non preparsing mode. We need to create the variable
      * unless someone want to check all var_Get/var_Change return value ... */
-    if( !p_input->b_preparsing )
+    if( !input_priv(p_input)->b_preparsing )
         InputAddCallbacks( p_input, p_input_callbacks );
 }
 
@@ -229,21 +226,25 @@ void input_ControlVarInit ( input_thread_t *p_input )
  *****************************************************************************/
 void input_ControlVarStop( input_thread_t *p_input )
 {
-    if( !p_input->b_preparsing )
+    if( !input_priv(p_input)->b_preparsing )
         InputDelCallbacks( p_input, p_input_callbacks );
 
-    if( p_input->p->i_title > 0 )
+    if( input_priv(p_input)->i_title > 1 )
+        InputDelCallbacks( p_input, p_input_title_navigation_callbacks );
+
+    for( int i = 0; i < input_priv(p_input)->i_title; i++ )
     {
-        InputDelCallbacks( p_input, p_input_navigation_callbacks );
-        InputDelCallbacks( p_input, p_input_title_callbacks );
+        char name[sizeof("title ") + 3 * sizeof (int)];
 
-        for( int i = 0; i < p_input->p->i_title; i++ )
-        {
-            char name[sizeof("title ") + 3 * sizeof (int)];
+        sprintf( name, "title %2u", i );
+        var_DelCallback( p_input, name, NavigationCallback, (void *)(intptr_t)i );
+    }
 
-            sprintf( name, "title %2u", i );
-            var_DelCallback( p_input, name, NavigationCallback, (void *)(intptr_t)i );
-        }
+    if( var_Type( p_input, "next-chapter" ) != 0 )
+    {
+        assert( var_Type( p_input, "prev-chapter" ) != 0 );
+        var_Destroy( p_input, "next-chapter" );
+        var_Destroy( p_input, "prev-chapter" );
     }
 }
 
@@ -256,7 +257,7 @@ void input_ControlVarNavigation( input_thread_t *p_input )
     vlc_value_t text;
 
     /* Create more command variables */
-    if( p_input->p->i_title > 1 )
+    if( input_priv(p_input)->i_title > 1 )
     {
         if( var_Type( p_input, "next-title" ) == 0 ) {
             var_Create( p_input, "next-title", VLC_VAR_VOID );
@@ -290,7 +291,7 @@ void input_ControlVarNavigation( input_thread_t *p_input )
     /* Create titles and chapters */
     var_Change( p_input, "title", VLC_VAR_CLEARCHOICES, NULL, NULL );
 
-    for( int i = 0; i < p_input->p->i_title; i++ )
+    for( int i = 0; i < input_priv(p_input)->i_title; i++ )
     {
         vlc_value_t val2, text2;
         char title[sizeof("title ") + 3 * sizeof (int)];
@@ -298,32 +299,31 @@ void input_ControlVarNavigation( input_thread_t *p_input )
         /* Add Navigation entries */
         sprintf( title, "title %2u", i );
         var_Destroy( p_input, title );
-        var_Create( p_input, title,
-                    VLC_VAR_INTEGER|VLC_VAR_HASCHOICE|VLC_VAR_ISCOMMAND );
+        var_Create( p_input, title, VLC_VAR_INTEGER | VLC_VAR_ISCOMMAND );
         var_AddCallback( p_input, title,
                          NavigationCallback, (void *)(intptr_t)i );
 
         char psz_length[MSTRTIME_MAX_SIZE + sizeof(" []")];
-        if( p_input->p->title[i]->i_length > 0 )
+        if( input_priv(p_input)->title[i]->i_length > 0 )
         {
             strcpy( psz_length, " [" );
-            secstotimestr( &psz_length[2], p_input->p->title[i]->i_length / CLOCK_FREQ );
+            secstotimestr( &psz_length[2], input_priv(p_input)->title[i]->i_length / CLOCK_FREQ );
             strcat( psz_length, "]" );
         }
         else
             psz_length[0] = '\0';
 
-        if( p_input->p->title[i]->psz_name == NULL ||
-            *p_input->p->title[i]->psz_name == '\0' )
+        if( input_priv(p_input)->title[i]->psz_name == NULL ||
+            *input_priv(p_input)->title[i]->psz_name == '\0' )
         {
             if( asprintf( &text.psz_string, _("Title %i%s"),
-                          i + p_input->p->i_title_offset, psz_length ) == -1 )
+                          i + input_priv(p_input)->i_title_offset, psz_length ) == -1 )
                 continue;
         }
         else
         {
             if( asprintf( &text.psz_string, "%s%s",
-                          p_input->p->title[i]->psz_name, psz_length ) == -1 )
+                          input_priv(p_input)->title[i]->psz_name, psz_length ) == -1 )
                 continue;
         }
 
@@ -333,22 +333,22 @@ void input_ControlVarNavigation( input_thread_t *p_input )
 
         free( text.psz_string );
 
-        for( int j = 0; j < p_input->p->title[i]->i_seekpoint; j++ )
+        for( int j = 0; j < input_priv(p_input)->title[i]->i_seekpoint; j++ )
         {
             val2.i_int = j;
 
-            if( p_input->p->title[i]->seekpoint[j]->psz_name == NULL ||
-                *p_input->p->title[i]->seekpoint[j]->psz_name == '\0' )
+            if( input_priv(p_input)->title[i]->seekpoint[j]->psz_name == NULL ||
+                *input_priv(p_input)->title[i]->seekpoint[j]->psz_name == '\0' )
             {
                 /* Default value */
                 if( asprintf( &text2.psz_string, _("Chapter %i"),
-                          j + p_input->p->i_seekpoint_offset ) == -1 )
+                          j + input_priv(p_input)->i_seekpoint_offset ) == -1 )
                     continue;
             }
             else
             {
                 text2.psz_string =
-                    strdup( p_input->p->title[i]->seekpoint[j]->psz_name );
+                    strdup( input_priv(p_input)->title[i]->seekpoint[j]->psz_name );
             }
 
             var_Change( p_input, title, VLC_VAR_ADDCHOICE, &val2, &text2 );
@@ -364,7 +364,7 @@ void input_ControlVarNavigation( input_thread_t *p_input )
  *****************************************************************************/
 void input_ControlVarTitle( input_thread_t *p_input, int i_title )
 {
-    input_title_t *t = p_input->p->title[i_title];
+    const input_title_t *t = input_priv(p_input)->title[i_title];
     vlc_value_t text;
     int  i;
 
@@ -399,7 +399,7 @@ void input_ControlVarTitle( input_thread_t *p_input, int i_title )
         {
             /* Default value */
             if( asprintf( &text.psz_string, _("Chapter %i"),
-                      i + p_input->p->i_seekpoint_offset ) == -1 )
+                      i + input_priv(p_input)->i_seekpoint_offset ) == -1 )
                 continue;
         }
         else
@@ -420,7 +420,7 @@ void input_ConfigVarInit ( input_thread_t *p_input )
 {
     /* Create Object Variables for private use only */
 
-    if( !p_input->b_preparsing )
+    if( !input_priv(p_input)->b_preparsing )
     {
         var_Create( p_input, "video", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
         var_Create( p_input, "audio", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
@@ -513,6 +513,7 @@ void input_ConfigVarInit ( input_thread_t *p_input )
     /* */
     var_Create( p_input, "access", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
     var_Create( p_input, "demux", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+    var_Create( p_input, "demux-filter", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
     var_Create( p_input, "stream-filter", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
 
     /* Meta */
@@ -585,32 +586,20 @@ static int PositionCallback( vlc_object_t *p_this, char const *psz_cmd,
                              void *p_data )
 {
     input_thread_t *p_input = (input_thread_t*)p_this;
-    VLC_UNUSED(oldval); VLC_UNUSED(p_data);
 
-    if( !strcmp( psz_cmd, "position-offset" ) )
+    VLC_UNUSED(psz_cmd); VLC_UNUSED(oldval); VLC_UNUSED(p_data);
+
+    /* Update "length" for better intf behaviour */
+    const int64_t i_length = var_GetInteger( p_input, "length" );
+    if( i_length > 0 && newval.f_float >= 0.f && newval.f_float <= 1.f )
     {
-        float f_position = var_GetFloat( p_input, "position" ) + newval.f_float;
-        if( f_position < 0.f )
-            f_position = 0.f;
-        else if( f_position > 1.f )
-            f_position = 1.f;
-        var_SetFloat( p_this, "position", f_position );
-    }
-    else
-    {
-        /* Update "length" for better intf behaviour */
-        const int64_t i_length = var_GetInteger( p_input, "length" );
-        if( i_length > 0 && newval.f_float >= 0.f && newval.f_float <= 1.f )
-        {
-            vlc_value_t val;
+        vlc_value_t val;
 
-            val.i_int = i_length * newval.f_float;
-            var_Change( p_input, "time", VLC_VAR_SETVALUE, &val, NULL );
-        }
-
-        /* */
-        input_ControlPush( p_input, INPUT_CONTROL_SET_POSITION, &newval );
+        val.i_int = i_length * newval.f_float;
+        var_Change( p_input, "time", VLC_VAR_SETVALUE, &val, NULL );
     }
+
+    input_ControlPush( p_input, INPUT_CONTROL_SET_POSITION, &newval );
     return VLC_SUCCESS;
 }
 
@@ -759,31 +748,50 @@ static int NavigationCallback( vlc_object_t *p_this, char const *psz_cmd,
     return VLC_SUCCESS;
 }
 
-static int ESCallback( vlc_object_t *p_this, char const *psz_cmd,
-                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+static int EsVideoCallback( vlc_object_t *p_this, char const *psz_cmd,
+                            vlc_value_t oldval, vlc_value_t newval, void *p_data )
 {
     input_thread_t *p_input = (input_thread_t*)p_this;
-    VLC_UNUSED(oldval); VLC_UNUSED(p_data);
+    VLC_UNUSED( psz_cmd); VLC_UNUSED( oldval ); VLC_UNUSED( p_data );
 
     if( newval.i_int < 0 )
-    {
-        vlc_value_t v;
-        /* Hack */
-        if( !strcmp( psz_cmd, "audio-es" ) )
-            v.i_int = -AUDIO_ES;
-        else if( !strcmp( psz_cmd, "video-es" ) )
-            v.i_int = -VIDEO_ES;
-        else if( !strcmp( psz_cmd, "spu-es" ) )
-            v.i_int = -SPU_ES;
-        else
-            v.i_int = 0;
-        if( v.i_int != 0 )
-            input_ControlPush( p_input, INPUT_CONTROL_SET_ES, &v );
-    }
+        newval.i_int = -VIDEO_ES; /* disable video es */
     else
-    {
-        input_ControlPush( p_input, INPUT_CONTROL_SET_ES, &newval );
-    }
+        var_SetBool( p_input, "video", true );
+
+    input_ControlPush( p_input, INPUT_CONTROL_SET_ES, &newval );
+
+    return VLC_SUCCESS;
+}
+
+static int EsAudioCallback( vlc_object_t *p_this, char const *psz_cmd,
+                            vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    input_thread_t *p_input = (input_thread_t*)p_this;
+    VLC_UNUSED( psz_cmd); VLC_UNUSED( oldval ); VLC_UNUSED( p_data );
+
+    if( newval.i_int < 0 )
+        newval.i_int = -AUDIO_ES; /* disable audio es */
+    else
+        var_SetBool( p_input, "audio", true );
+
+    input_ControlPush( p_input, INPUT_CONTROL_SET_ES, &newval );
+
+    return VLC_SUCCESS;
+}
+
+static int EsSpuCallback( vlc_object_t *p_this, char const *psz_cmd,
+                           vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    input_thread_t *p_input = (input_thread_t*)p_this;
+    VLC_UNUSED( psz_cmd); VLC_UNUSED( oldval ); VLC_UNUSED( p_data );
+
+    if( newval.i_int < 0 )
+        newval.i_int = -SPU_ES; /* disable spu es */
+    else
+        var_SetBool( p_input, "spu", true );
+
+    input_ControlPush( p_input, INPUT_CONTROL_SET_ES, &newval );
 
     return VLC_SUCCESS;
 }
