@@ -1,7 +1,7 @@
 /*
  * test.h - libvlc smoke test common definitions
  *
- * $Id: 4e8356a07db545838ac886855690b658a5b29cdf $
+ * $Id: 62e260fd89847a4690cfe40f73acae31ff0e2335 $
  */
 
 /**********************************************************************
@@ -48,7 +48,7 @@
  */
 
 static const char * test_defaults_args[] = {
-    "-v",
+    "-v", "--vout=vdummy",
 };
 
 static const int test_defaults_nargs =
@@ -56,6 +56,7 @@ static const int test_defaults_nargs =
 
 /*static const char test_default_sample[] = "samples/test.sample";*/
 static const char test_default_sample[] = SRCDIR"/samples/empty.voc";
+static const char test_default_video[] = SRCDIR"/samples/image.jpg";
 
 
 /*********************************************************************
